import os
import re
import pandas as pd
import seaborn as sns
import warnings
import matplotlib.pyplot as plt
from .YModulus import YModulus

def multiple2(allname='Result', polymers=[], loc='./', savefig=True,\
              savedata=True, xx=False):
    """
    The method to create a Distance-Force chart and .csv relevant data file of all polymers you choose.
    multiple2(allname='Results', polymers=[file1, file2, file3, ...], loc='./', savefig=True, savedata=True, needYM=True))
    allname: The name of your saved Distance-Force curve chart and data file.
    polymers: Your selected polymer names on your file. The default is all polymers which has _Results.txt file in the location. 
    loc: File Location. The default is your current location.
    savefig, savedata: To show if you want to save the chart and .csv curve. The default is True.
    neddYM: To calculate Young's modulus of each polymers and display it in the data file.
    You can get the further information by .curve and .autocurve.
    Example:
        Input:
            from MCPoly.sscurve import multiple
            multiple2(polymers=['P1', 'P2'])
        Output:
            ['P1', 'P2']
                               Distance (P1)         Distance (P2)  \
            Force(nN)                                            
            0.0                         1.000000             1.000000   
            0.5                         1.012022                  NaN   
            1.0                         1.024666             1.028089   
            1.5                              NaN                  NaN   
            2.0                         1.053014             1.058092   
            2.5                              NaN                  NaN   
            3.0                         1.086888             1.093080   
            3.5                              NaN                  NaN   
            4.0                         1.130600             1.136959   
            4.5                              NaN                  NaN   
            5.0                         1.199289             1.203102   
            5.5                              NaN                  NaN   
            6.0                              NaN                  NaN   
            6.5                              NaN                  NaN   
            7.0                              NaN                  NaN   
            7.5                              NaN                  NaN   
            8.0                              NaN                  NaN   
            8.5                              NaN                  NaN   
            9.0                              NaN                  NaN   
            9.5                              NaN                  NaN   
            10.0                             NaN                  NaN   
            NaN                        29.516401             23.478160  NaN-> Young's Modulus
    TIPS: This function only depends on _Results.txt file, not .mol, .inp, .out, .xyz files.
    """
    opath = os.getcwd()
    if polymers == []:
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('_Result.txt', path)
                if a:
                    polymers.append(path[:-11])            
    print(polymers)
    
    os.chdir(loc)
    try:
        f = open('ot.txt', 'x')
    except:
        f = open('ot.txt', 'w')
    
    name = []
    i = 0
    
    site = []

    for polymer in polymers:
        polymer = polymer + '_Result.txt'
        try:
            datum = pd.read_csv(polymer)
        except:
            warnings.warn("'{0}' is not found.".format(polymer))
            continue
        datum = datum[:].sort_values(by=['Force(nN)'])
        for num in datum['Force(nN)'][:-1]:
            if num not in site:
                site.append(num)
    site.sort()
    f.write('Force(nN)\n')
    for num in site:
        f.write('{0:.3f}\n'.format(num))
    f.close()
    ot = pd.read_csv('ot.txt')
    for polymer in polymers:
        polymer = polymer + '_Result.txt'
        try:
            datum = pd.read_csv(polymer)
        except:
            continue
        datum = datum[:].sort_values(by=['Distance(Å)'])
        if xx == False:
            ax = plt.plot(datum['Force(nN)'][:-1],\
                          datum['Distance(Å)'][:-1], '*-')
        datum = datum.rename(
            columns={'Distance(Å)':'Distance(Å)({0})'.format(
                polymer[:-11])})
        ot = pd.merge_ordered(ot, datum[:-1], fill_method="ffill",\
                              left_by="Force(nN)")
        name.append(polymer[:-11])
        i = i + 1
    
    ot = ot.set_index('Force(nN)')
    display(ot)
    if savedata == True:
        ot.to_csv('{0}.csv'.format(allname))
    
    if xx == False:
        #ax = sns.scatterplot(data=ot, alpha=0.8)
        #ax.set_ylabels='Distance(Å)'
        #_ = plt.title('Distance-Force Relationship')
        #bx = sns.relplot(data=ot, kind="line", alpha=0.5)
        #bx.set_ylabels='Distance(Å)'
        #ax.legend(bbox_to_anchor=[1.1, 0.9])
        plt.xlabel('Force(nN)')
        plt.ylabel('Distance(Å)')
        plt.legend(name, bbox_to_anchor=[1.1, 0.9])
        _ = plt.title('Distance-Force Relationship')
    
    if savefig == True:
        plt.savefig('{0}.png'.format(allname), bbox_inches='tight')
    if xx == False:
        plt.show()
    
    os.system('rm ot.txt')
    os.chdir(opath)


import sys
import os
import re
import ipywidgets as widgets
from ase.io import read
from ase import Atoms

mydir = os.path.dirname( __file__ )
orcadir = os.path.join(mydir, '..', 'orcaset')
sys.path.append(orcadir)

from view3dchoose import view3dchoose
from XyzToInp import XyzToInp
from orca import orca
import time as t

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

mydir = os.path.dirname( __file__ )
viewdir = os.path.join(mydir, '..', 'view3d')
sys.path.append(viewdir)
from view3d import view3d

mydir = os.path.dirname( __file__ )
ssdir = os.path.join(mydir, '..', 'sscurve')
sys.path.append(ssdir)
from single import single

def ssgui():
    """
        The method to use GUI to set ORCA input files, calculate the file with external force by ORCA and draw stress-strain curve in handy!
        You will know how to do as long as you get into the GUI platform.

        TIPS: 1. This function can calculate ORCA input files, and draw single stress-strain curve and save data texts as well.
              2. Take caution when you set External Force site. Don't set number out of range.
              3. To see the status of your optimisation, you can set another window and use status.figuretraj .
              4. The visualisation will also risk the flexibility of the task, to make you programme more flexible, please directly use the code.
    """
    output = widgets.Output()
    output2 = widgets.Output()
    addoutput1 = widgets.Output()
    addoutput2 = widgets.Output()
    statusoutput = widgets.Output()
    geooutput = widgets.Output()
    curveoutput = widgets.Output()
    fromoutput = widgets.Output()
    case = widgets.Output()

    def checkbroken(distance, force):
        long1 = abs(distance[-2] - distance[-1])
        long2 = abs(distance[-3] - distance[-2])
        mul1 = abs(force[-2] - force[-1])
        mul2 = abs(force[-3] - force[-2])
        if long1 / long2 >= 10 * mul1 / mul2:
            return 1
        else:
            return 0
    
    def changecombo(loc):
        options2 = []
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('.out', path)
                if a:
                    options2.append(path[:-4])
        options2.sort()
        file.options = options2

    def showfrom(fromshow, file, loc, aim1, aim2):
        fromoutput.clear_output()
        curveoutput.clear_output()
        curve.value = None
        opath = os.getcwd()
        polymers0 = []
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.match('{0}_'.format(file), path)
                if a:
                    b = re.search('_trj.xyz', path)
                    if b:
                        c = re.match('{0}_trj.xyz'.format(file), path)
                        if not c:
                            polymers0.append(path[:-8])
        polynums = []
        polymers = []
        polymers0.sort()
        for polymer in polymers0:
            a = re.split('_', polymer)
            polynums.append(eval(a[1]))
            polyname = a[0]
        polynums.sort()
        
        for num in polynums:
            polymers.append('{0}_{1:.3f}'.format(polyname, num))
        
        with fromoutput:
            for i, polymer in enumerate(polymers):
                print(polymer+': {0:.6f} Hartree'.format(status(polymer,\
                                                                loc).status(figureonly = True)[-1]))
                if fromshow == False:
                    view3d(polymer, loc, width=450, height=400)
                else:
                    view3dchoose(polymer, loc, choose=[aim1, aim2], width=450, height=400)
        os.chdir(opath)

    def curveshow(curve, file, loc):
        output2.clear_output()
        curveoutput.clear_output()
        case.clear_output()
        with curveoutput:
            if file == '':
                return None
            if curve == 'Stress-Strain Curve':
                single(file, loc).autocurve(savefig=False, savedata=False)
                display(widgets.HBox([savechart, savedata]))
            elif curve == 'Distance-Force Curve':
                single(file, loc).autocurve2(savefig=False, savedata=False)
                display(widgets.HBox([savechart, savedata]))
        
#    def aborting(button):
#        

    def addshow1(addox1):
        addoutput1.clear_output()
        if addox1 == True:
            addox2.disabled = False
            with addoutput1:
                display(strain2, addforce2)
        elif addox1 == False:
            addox2.value=False
            addox2.disabled = True

    def addshow2(addox2):
        addoutput2.clear_output()
        if addox2 == True:
            with addoutput2:
                display(strain3, addforce3)

    def geoshow(file, loc, aim1, aim2, method, bs):
        aa.value=0
        atomsx = read(loc+file+'.xyz')
        num = len(atomsx)
        output.clear_output()
        with output:
            try:
                if aim1 == 0 and aim2 == 0:
                    view3d(file, loc, width=450, height=400)
                else:
                    if aim1 >= num or aim2 >= num:
                        print('Setting Error. The maximum number of this system is {0}.\n We will automatically change the number into the maximum number.'.format(num-1))
                    if aim1 >= num:
                        aim1 = num - 1
                    if aim2 >= num:
                        aim2 = num - 1
                    aim=[aim1, aim2]
                    print('Atom1: {0}      Atom2: {1}'.format(aim1, aim2))
                    view3dchoose(file, loc, choose=aim, width=450, height=400)
            except:
                if file == '':
                    print('Please input the file to show the geometry structure on this screen.')
                else:
                    print('No such file.')
            if method != '' and bs != '':
                display(widgets.HBox([widgets.Label('ORCA Location:'), orcaloc]), startorca)
    
    def savechartx(button):
        if curve.value == 'Stress-Strain Curve':
            single(file.value, loc.value).autocurve(savedata=False)
        elif curve.value == 'Distance-Force Curve':
            single(file.value, loc.value).autocurve2(savedata=False)
        
    def savedatax(button):
        if curve.value == 'Stress-Strain Curve':
            single(file.value, loc.value).autocurve(savefig=False)
        elif curve.value == 'Distance-Force Curve':
            single(file.value, loc.value).autocurve2(savefig=False)
    
    def intro_status3d(button):
        curve.value = None
        atomsx = read(loc.value+file.value+'.xyz')
        global forcestep
        global key
        num = len(atomsx)
        if aim1.value >= num:
            aim1.value = num - 1
        if aim2.value >= num:
            aim2.value = num - 1
        output.clear_output()
        case.clear_output()
        if aa.value == 404:
            with output2:
                if roomox.value == False and coreox.value == False:
                    XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                             inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                             method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                             external_force=True, aim=[aim1.value, aim2.value],\
                             strain = strain.value + forcestep[key],\
                             maxiter=maxiter.value, maxcore=room.value*1024, corenum=corenum.value)
                elif roomox.value == True and coreox.value == True:
                    XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                             inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                             method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                             external_force=True, aim=[aim1.value, aim2.value],\
                             strain = strain.value + forcestep[key], maxiter=maxiter.value)
                elif roomox.value == False and coreox.value == True:
                    XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                             inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                             method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                             external_force=True, aim=[aim1.value, aim2.value],\
                             strain = strain.value + forcestep[key], maxiter=maxiter.value,\
                             maxcore=room.value*1024)
                elif roomox.value == True and coreox.value == False:
                    XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                             inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                             method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                             external_force=True, aim=[aim1.value, aim2.value],\
                             strain = strain.value + forcestep[key],\
                             maxiter=maxiter.value, corenum=corenum.value)
            strain.value = strain.value + forcestep[key]
            keys = []
            if addforce.value == 0:
                raise ValueError("You can't set the force add per step zero!")
            forcestep = {strain.value: addforce.value}
            if addox1.value == True:
                if addforce2.value == 0:
                    raise ValueError("You can't set the force add per step zero!")
                forcestep = {strain.value: addforce.value, strain2.value: addforce2.value}
            elif addox1.value == True and addox2.value == True:
                if addforce3.value == 0:
                    raise ValueError("You can't set the force add per step zero!")
                forcestep = {strain.value: addforce.value, strain2.value: addforce2.value,\
                             strain3.value: addforce3.value}
            for key in forcestep:
                keys.append(key)
            keys.reverse()
            key = keys[0]
            ii = 2
        else:
            output2.clear_output()
            with output2:
                display(widgets.Label('File: {0}.xyz'.format(file.value),\
                                      style = dict(font_weight='bold')))
                if roomox.value == False and coreox.value == False:
                    XyzToInp(file.value, fileloc = loc.value, method=method.value,\
                             basis_set=bs.value, opt=True, freq=freq.value, \
                             external_force=True, aim=[aim1.value, aim2.value],\
                             strain=strain.value, maxiter=maxiter.value,\
                             maxcore=room.value*1024, corenum=corenum.value)
                elif roomox.value == True and coreox.value == True:
                    XyzToInp(file.value, fileloc = loc.value, method=method.value,\
                             basis_set=bs.value, opt=True, freq=freq.value, \
                             external_force=True, aim=[aim1.value, aim2.value],\
                             maxiter=maxiter.value, strain=strain.value)
                elif roomox.value == False and coreox.value == True:
                    XyzToInp(file.value, fileloc = loc.value, method=method.value,\
                             basis_set=bs.value, opt=True, freq=freq.value,\
                             external_force=True, aim=[aim1.value, aim2.value],\
                             maxiter=maxiter.value, strain=strain.value, maxcore=room.value*1024)
                elif roomox.value == True and coreox.value == False:
                    XyzToInp(file.value, fileloc = loc.value, method=method.value,\
                             basis_set=bs.value, opt=True, freq=freq.value, \
                             external_force=True, aim=[aim1.value, aim2.value],\
                             maxiter=maxiter.value, strain=strain.value, corenum=corenum.value)
            if strain.value == 0.000:
                os.system('cp {0}.inp {0}_0.000.inp'.format(file.value))
            keys = []
            if addforce.value == 0:
                raise ValueError("You can't set the force add per step zero!")
            forcestep = {strain.value:addforce.value}
            if addox1.value == True:
                if addforce2.value == 0:
                    raise ValueError("You can't set the force add per step zero!")
                forcestep = {strain.value: addforce.value, strain2.value: addforce2.value}
            elif addox1.value == True and addox2.value == True:
                if addforce3.value == 0:
                    raise ValueError("You can't set the force add per step zero!")
                forcestep = {strain.value: addforce.value, strain2.value: addforce2.value,\
                             strain3.value: addforce3.value}
            for key in forcestep:
                keys.append(key)
            keys.reverse()
            key = keys[0]
            ii = -1
            global distances
            distances = []
            global force
            force = []
        while 1:
            with case:
                display(widgets.HBox([
                    widgets.Label(file.value+'_{0:.3f}'.format(strain.value)),
                    widgets.Label('  Processing...'), 
                    widgets.Label('[{0}]'.format(t.ctime(t.time()))),
                ]))
            orca(file.value+'_{0:.3f}'.format(strain.value),\
                 orcaloc.value, loc.value, loc.value)
            ii = ii + 1
            s = status(file.value+'_{0:.3f}'.format(strain.value),\
                       loc.value).status(statusonly = True)
            #print(s)
            the_atoms = read(file.value+'_{0:.3f}.xyz'.format(strain.value))
            d = the_atoms.get_distance(aim1.value, aim2.value)
            case.clear_output()
            if s == 4 or s == 41:
                aa.value = 0
                distances.append(d)
                force.append(strain.value)
                xoy = checkbroken(distances, force)
                if xoy == 1 and s != 41:
                    with output2:
                        display(widgets.HBox([
                            widgets.Valid(description = file.value
                                          + '_{0:.3f}'.format(strain.value), value=True),\
                            widgets.Label('Broken.'),\
                            widgets.Label('[{0}]'.format(t.ctime(t.time())))]))
                    fromoutput.clear_output()
                    with fromoutput:
                        for i, poly in enumerate(polys):
                            print(poly+': {0:.6f} Hartree'.format(status(poly,\
                                                                         loc.value).status(figureonly=True)[-1]))
                            if fromshow == False:
                                view3d(poly, loc.value, width=450, height=400)
                            else:
                                view3dchoose(poly, loc.value,\
                                             choose = [aim1.value, aim2.value], width=450, height=400)
                    break
                else:
                    case.clear_output()
                    with case:
                        display(widgets.Valid(description = file.value
                                              + '_{0:.3f}'.format(strain.value),\
                                              value=False, readout = 'Retrying...'),\
                                widgets.Label('[{0}]'.format(t.ctime(t.time()))))
                    orca(file.value+'_{0:.3f}'.format(strain.value),\
                         orcaloc.value, loc.value, loc.value)
                    s = status(file.value, loc.value).status(statusonly=True)
                    if s == 4 or s == 41:
                        distances[-1] = None
                        case.clear_output()
                        with case:
                            display(widgets.Valid(description = file.value
                                                  + '_{0:.3f}'.format(strain.value),\
                                                  value=False, readout = 'Error!'),\
                                    widgets.Label('[{0}]'.format(t.ctime(t.time()))))
                    else:
                        distances[-1] = d
            elif s == 2:
                aa.value = 0
                with output2:
                    distances.append(d)
                    force.append(strain.value)
                    display(widgets.HBox([widgets.Valid(description = file.value
                                                        + '_{0:.3f}'.format(strain.value), value=True),\
                                          widgets.Label('Completed.'),\
                                          widgets.Label('[{0}]'.format(t.ctime(t.time())))]))
            elif s == 6:
                aa.value = 404
                with output2:
                    distances.append(d)
                    force.append(strain.value)
                    display(widgets.HBox([widgets.Valid(description = file.value
                                                        + '_{0:.3f}'.format(strain.value), value=True), \
                                          widgets.Label('Not Converged.'),\
                                          widgets.Label('[{0}]'.format(t.ctime(t.time())))]))
                with case:
                    display(widgets.Label("{0}_{1:.3f} is finished but\
                        it's not converged.\n Continue?".format(file.value, strain.value)))
                    display(widgets.VBox([stop, curve]))
                #strain.value = strain.value + forcestep[key]
                break
            if ii == 0:
                output.clear_output()
                with output:
                    display(fromoutput)
            
            opath = os.getcwd()
            polys = []
            try:
                for path in os.listdir(loc.value):
                    if os.path.isfile(os.path.join(loc.value, path)):
                        a = re.match('{0}_'.format(file.value), path)
                        if a:
                            b = re.search('_trj.xyz', path)
                            if b:
                                c = re.search('{0}_trj.xyz'.format(file.value), path)
                                if not c:
                                    polys.append(path[:-8])
                polys.sort()
                fromoutput.clear_output()
                with fromoutput:
                    for i, poly in enumerate(polys):
                        print(poly+': {0:.6f} Hartree'.format(status(poly,\
                                                                     loc.value).status(figureonly = True)[-1]))
                        if fromshow == False:
                            view3d(poly, loc.value, width=450, height=400)
                        else:
                            view3dchoose(poly, loc.value, choose=[aim1.value, aim2.value],\
                                         width=450, height=400)
            except:
                pass
            os.chdir(opath)
            
            for key in keys:
                if strain.value >= key:
                    if roomox.value == False and coreox.value == False:
                        XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                                 inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                                 method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                                 external_force=True, aim=[aim1.value, aim2.value],\
                                 strain=strain.value+forcestep[key], maxiter=maxiter.value,\
                                 maxcore=room.value*1024, corenum=corenum.value)
                    elif roomox.value == True and coreox.value == True:
                        XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                                 inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                                 method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                                 external_force=True, aim=[aim1.value, aim2.value],\
                                 strain=strain.value+forcestep[key], maxiter=maxiter.value)
                    elif roomox.value == False and coreox.value == True:
                        XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                                 inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                                 method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                                 external_force=True, aim=[aim1.value, aim2.value],\
                                 strain=strain.value+forcestep[key],\
                                 maxiter=maxiter.value, maxcore=room.value*1024)
                    elif roomox.value == True and coreox.value == False:
                        XyzToInp(file.value+'_{0:.3f}'.format(strain.value),\
                                 inpname = file.value+'_{0:.3f}'.format(strain.value+forcestep[key]),\
                                 method=method.value, basis_set=bs.value, opt=True, freq=freq.value,\
                                 external_force=True, aim=[aim1.value, aim2.value],\
                                 strain=strain.value+forcestep[key],\
                                 maxiter=maxiter.value, corenum=corenum.value)
                    strain.value = strain.value + forcestep[key]
                    break
        if aa.value != 404:
            with output:
                display(fromshow)
            with output2:
                display(curve)

    def statusresult(button):
        statusoutput.clear_output()
        with statusoutput:
            while 1:
                if s == 1:
                    break
                t.sleep(30)
    
    def aashow(aa):
        if aa == 404:
            t.sleep(5)
            intro_status(stop)
            
    options = []
    loc = widgets.Text(description = 'Location:', value='./')
    for path in os.listdir(loc.value):
        if os.path.isfile(os.path.join(loc.value, path)):
            a = re.search('.xyz', path)
            if a:
                options.append(path[:-4])
    file = widgets.Combobox(placeholder = 'Input the file name', options=options, 
         description = 'xyz File:')
    ssfiles = []
    for path in os.listdir(loc.value):
        if os.path.isfile(os.path.join(loc.value, path)):
            a = re.search('.xyz', path)
            if a:
                b = re.search(file.value, path)
                if b:
                    ssfiles.append(path[:-4])

    aa = widgets.IntText(value=0)
    method = widgets.Text(description = 'Method:')
    bs = widgets.Text(description = 'Basis Set:')
    freq=widgets.ToggleButton(value=False, description = 'Frequency',\
                              layout = widgets.Layout(width='33.3%'))
    room = widgets.IntText(description = 'Space(GB):', value=4, min=1,\
                           layout = widgets.Layout(width='50%'))
    roomox = widgets.ToggleButton(value=False, description = 'Default Space')
    corenum = widgets.IntText(description = 'Core:', value=4, min=1,\
                              layout = widgets.Layout(width='50%'))
    coreox = widgets.ToggleButton(value=False, description = 'Default CoreNumber')
    aim1 = widgets.BoundedIntText(description = 'Atom 1:', value=0,\
                                  min=0, max=300, layout = widgets.Layout(width='50%'))
    aim2 = widgets.BoundedIntText(description = 'Atom 2:', value=0,\
                                  min=0, max=300, layout = widgets.Layout(width='47.5%'))
    strain = widgets.BoundedFloatText(description = 'Start Force(nN):',\
                                    value=0.000, min=0.000, step = 0.100,\
                                    style = {'description_width': 'initial'})
    addforce = widgets.BoundedFloatText(description = 'Force Add per Step(nN):',\
                                        value=0.100, min=0.000, step = 0.100,\
                                        style = {'description_width': 'initial'})
    addox1 = widgets.ToggleButton(value=False, description = 'The Second Stage',\
                                  style = dict(font_weight='bold'))
    strain2 = widgets.BoundedFloatText(description = 'Till Force(nN):',\
                                       value=0.000, min=0.000, step = 0.100,\
                                       style = {'description_width': 'initial'})
    addforce2 = widgets.BoundedFloatText(description = 'Force Add per Step(nN):',\
                                         value=0.100, min=0.000, step = 0.100,\
                                         style = {'description_width': 'initial'})
    addox2 = widgets.ToggleButton(value=False, description = 'The Third Stage',\
                                  style = dict(font_weight='bold'))
    strain3 = widgets.BoundedFloatText(description = 'Till Force(nN):',\
                                       value=0.000, min=0.000, step = 0.100,\
                                       style = {'description_width': 'initial'})
    addforce3 = widgets.BoundedFloatText(description = 'Force Add per Step(nN):',\
                                         value=0.100, min=0.000, step = 0.100,\
                                         style = {'description_width': 'initial'})
    startorca = widgets.Button(description = 'Start ORCA Calculation',\
                               layout = widgets.Layout(width='99%'),\
                               style = dict(font_weight='bold', text_decoration='underline'))
    orcaloc = widgets.Text(value='./')
    stop = widgets.Button(description = 'Continue!',\
                          layout = widgets.Layout(width='99%'))
    curve = widgets.ToggleButtons(options = ['Stress-Strain Curve', 'Distance-Force Curve'],\
                                  description = 'Curve:',\
                                  layout = widgets.Layout(width='99%'), value=None, \
                                  style = dict(font_weight='bold', text_decoration='underline'))
    fromshow = widgets.ToggleButton(description = 'Show the site of Force',\
                                    value=True, layout = widgets.Layout(width='99%'),\
                                    style = dict(font_weight='bold', text_decoration='underline'))
    savechart = widgets.Button(description = 'Save chart ( .png)',\
                               layout = widgets.Layout(width='50%'))
    savedata = widgets.Button(description = 'Save data (.txt)',\
                            layout = widgets.Layout(width='47.5%'))
    maxiter = widgets.IntText(description = 'MaxIteration:',\
                              value=200, layout = widgets.Layout(width='98.5%'))
    
    startorca.on_click(intro_status3d)
    savechart.on_click(savechartx)
    savedata.on_click(savedatax)
    stop.on_click(intro_status3d)
    
    title1 = widgets.Label('File Details', style = dict(font_weight='bold'))
    title2 = widgets.Label('Calculation Methods',\
                           style = dict(font_weight='bold'))
    title3 = widgets.Label('Constant External Force Settings',\
                           style = dict(font_weight='bold'))
    
    changecomboout = widgets.interactive_output(changecombo, {'loc': loc})
    addout1 = widgets.interactive_output(addshow1, {'addox1': addox1})
    addout2 = widgets.interactive_output(addshow2, {'addox2': addox2})
    geoout = widgets.interactive_output(geoshow, {'file': file, 'loc': loc,\
                                                  'aim1': aim1, 'aim2': aim2, 'method': method, 'bs': bs})
    ssout = widgets.interactive_output(curveshow, {'curve': curve, 'file': file, 'loc': loc})
    fromout = widgets.interactive_output(showfrom, {'fromshow': fromshow,\
                                                    'file': file, 'loc': loc, 'aim1': aim1, 'aim2': aim2})
    aacontinue = widgets.interactive_output(aashow, {'aa': aa})
    
    widgets.jslink((roomox, 'value'), (room, 'disabled'))
    widgets.jslink((coreox, 'value'), (corenum, 'disabled'))
    
    box_layout1 = widgets.Layout(display='flex', flew_flow='column',\
                                 overflow='scroll hidden', align_items='stretch',\
                                 border='solid', height='650px', width='31%')
    box_layout2 = widgets.Layout(display='flex', flew_flow='column',\
                                 overflow='scroll hidden',align_items='stretch',\
                                 border='solid', height='650px', width='33%')
    
    frame1 = widgets.VBox([title1, loc, file, widgets.HBox([title2, freq]),\
                           method, bs, widgets.HBox([corenum, coreox]),\
                           widgets.HBox([room, roomox]), maxiter, title3,\
                           widgets.HBox([aim1, aim2]), strain, addforce,\
                           addox1, addoutput1, addox2, addoutput2], layout=box_layout1)
    frame3 = widgets.VBox([output2, case, curveoutput])
    output.layout = box_layout2
    frame3.layout = box_layout2
    curveoutput.layout = widgets.Layout(display='flex', flew_flow='column',\
                                        overflow='scroll hidden', align_items='stretch',\
                                        height='650px', width='100%')
    output2.layout = widgets.Layout(display='flex', flew_flow='column',\
                                    overflow='scroll hidden', align_items='stretch', width='100%')
    display(widgets.HBox([frame1, output, frame3]))


import os
import re
import sys
import warnings
import math as m
import numpy as np
from ase.io import read, write
from ase import Atoms

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

def rebuild(file, substitute, x = [99999, 99999], y = [99999, 99999],\
            z = [99999, 99999], loc = './', subloc = './'):
    """
        The method to replace the geometry structure of LAMMPS Input File from an XYZ File.
        rebuild(file, substitute, x = [xmin, xmax], y = [ymin, ymax], z = [zmin, zmax], loc = './', subloc = './')
        file: Your molecule system name on your .data file.
        substitute: XYZ File used to replace.
        x, y, z: Caption of the box.
        loc: Data File Location. The default is your current location.
        subloc: XYZ File Location. The default is your current location.
        Example:
            Input:
                from MCPoly.lmpset import DATAtoXYZ
                from MCPoly.lmpset import rebuild
                from MCPoly.lmpset import mould
                
                atoms = mould('Atoms1')
                atoms.cube(1, 0, 1, 0, 2, 0) # Create the file Atoms1_112.data, see in lmpset.cube.
                DATAtoXYZ('Atoms1_112') # Create Atoms1_112.xyz, see in lmpset.DATAtoXYZ
                # After changing the geometry structure by GaussView or Avogrado and save it as Atoms1_replace.xyz
                rebuild('Atoms1_112', 'Atoms1_replace')
            
            Atoms1_replace.xyz:
                106
                
                O          9.64263        4.67845        3.83021
                C          9.97863        4.59047        2.66070
                N         11.06345        5.23157        2.07395
                O          9.31893        3.82840        1.73881
                C         12.08548        6.09614        2.75528
                H         11.25704        4.90681        1.13587
                C          8.15829        2.96807        2.00836
                ......
            
            Output in Atoms1_112.data:
                ......
                
                Atoms
                
                1 1 1 -0.4804 9.64263 4.67845 3.83021
                2 1 2 0.7694 9.97863 4.59047 2.66070
                3 1 3 -1.1201 11.06345 5.23157 2.07395
                4 1 4 -0.3798 9.31893 3.82840 1.73881
                5 1 5 0.1753 12.08548 6.09614 2.75528
                6 1 6 0.5150 11.25704 4.90681 1.13587
                7 1 5 0.0718 8.15829 2.96807 2.00836
                ......
            
    """
    path = os.getcwd()
    atoms = status('{0}'.format(substitute), loc = subloc)
    pos = atoms.atom()
    xmax = -99999
    xmin = 99999
    ymax = -99999
    ymin = 99999
    zmax = -99999
    zmin = 99999
    for num in pos:
        if num[1]  >  xmax:
            xmax = num[1]
        if num[2] > ymax:
            ymax = num[2]
        if num[3] > zmax:
            zmax = num[3]
        if num[1] < xmin:
            xmin = num[1]
        if num[2] < ymin:
            ymin = num[2]
        if num[3] < zmin:
            zmin = num[3]
    atoms_number = len(pos)
    os.chdir(path)
    readfile = open(loc+'{0}.data'.format(file), 'r')
    try:
        f = open(loc+'pre.data', 'x')
    except:
        f = open(loc+'pre.data', 'w')
    c = 0
    i = 0
    for line in readfile:
        x0 = re.search(' xlo xhi', line)
        y0 = re.search(' ylo yhi', line)
        z0 = re.search(' zlo zhi', line)
        a = re.search('Atoms', line)
        if x0:
            if x == [99999, 99999]:
                f.write('{0:.5f} {1:.5f} xlo xhi\n'.format(xmin, xmax))
            else:
                f.write('{0:.5f} {1:.5f} xlo xhi\n'.format(x[0], x[1]))
        elif y0:
            if y == [99999, 99999]:
                f.write('{0:.5f} {1:.5f} ylo yhi\n'.format(ymin, ymax))
            else:
                f.write('{0:.5f} {1:.5f} ylo yhi\n'.format(y[0], y[1]))
        elif z0:
            if z == [99999, 99999]:
                f.write('{0:.5f} {1:.5f} zlo zhi\n'.format(zmin, zmax))
            else:
                f.write('{0:.5f} {1:.5f} zlo zhi\n'.format(z[0], z[1]))
        elif c == 1:
            need = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(need) >= 6:
                c = 2
            if len(need) == 7:
                f.write('{0} {1} {2} {3} {4:.5f} {5:.5f} {6:.5f}\n'.format(need[0],\
                                                                           need[1], need[2], need[3], pos[i][1], pos[i][2], pos[i][3]))
                i = i + 1
            elif len(need) == 6:
                f.write('{0} {1} {2} {3:.5f} {4:.5f} {5:.5f}\n'.format(need[0],\
                                                                       need[1], need[2], pos[i][1], pos[i][2], pos[i][3]))
                i = i + 1
        elif c == 2:
            need = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(need) == 7:
                f.write('{0} {1} {2} {3} {4:.5f} {5:.5f} {6:.5f}\n'.format(need[0],\
                                                                           need[1], need[2], need[3], pos[i][1], pos[i][2], pos[i][3]))
            elif len(need) == 6:
                f.write('{0} {1} {2} {3:.5f} {4:.5f} {5:.5f}\n'.format(need[0],\
                                                                       need[1], need[2], pos[i][1], pos[i][2], pos[i][3]))
            i = i + 1
            if atoms_number == i:
                c = 0
        elif a:
            c = 1
            f.write(line)
            f.write('\n')
        else:
            f.write(line)
    readfile.close()
    f.close()
    opath = os.getcwd()
    os.chdir(loc)
    os.system('rm {0}.data'.format(file))
    os.system('cp pre.data {0}.data'.format(file))
    os.system('rm pre.data')
    os.chdir(path)


import os
import re
import sys
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

def echart(files=[], loc='./', energy_pattern='Energy', hartree=False,\
           absolute=False, fig_pattern='line', figdata=True, savefig=True,\
           figname='Result', savedata=True, dataname='Result', xx=False):
    """
        A method to make a chart with calculated Electronic Energy datas or Gibbs Free Energy data.
        echart(files=[], loc='./', energy_pattern='Energy', hartree=False, absolute=False, figname='Result', fig_pattern='line', savefig=False, dataname='Result', savedata=True)
        files: The name of ORCA output files already calculated.
        loc: File Location. The default is your current location.
        energy_pattern: Energy you need. There are 2 options, 'Energy' for normal Electronic Energy and 'Gibbs' for Gibbs Free Energy. If you want to use 'Gibbs', please make sure your output file including frequency and thermodynamics calculation.
        hartree: Energy unit is normally kcal/mol, if you need unit 'Hartree', please input 'hartree=True'
                 TIPS: 1 Hartree ≈ 627.509 kcal/mol
        absolute: Normally we'll automatical use Relative Energy to show energy change, if you want Absolute Energy, please input 'absolute=True'
        fig_pattern: Pattern of the figure. There are 2 options, 'line' means Line Chart and 'bar' means Bar Chart.
        figdata: Show the File Name and Energy on the chart. The defaulr is True.
        savefig / savedata: To show if you want to save the chart and .txt result. The default is True.
        figname / savename: To set the name of the chart and .txt result. The default is 'Result'. 
        TIPS: If you use it to collect Gibbs Free Energy, please make sure you have frequency calculation. If the programme can't find out thermodynamics calculation, it will return None(NaN).
        
        Example:
            Input:
                from MCPoly.status import echart
                a = echart(files = ['Molecule1', 'Molecule2', 'Molecule3', 'Molecule4', 'Molecule5', 'Molecule6'], fig_pattern='line', \
                    figdata=True)
                print(a)
            Output:
                <<matplotlib.plot>>
                [ 0.          0.79379889  3.3396029   8.29064891 16.99670877 33.95325697]
            In Result.csv:
                Step, File, ∆E(kcal/mol)
                0, Molecule1, 0.0
                1, Molecule2, 0.7937988850646964
                2, Molecule3, 3.3396028980577155
                3, Molecule4, 8.290648908041941
                4, Molecule5, 16.996708774033824
                5, Molecule6, 33.95325697202637
        
        TIPS: For some limitation, the chart created on this function might not very good.
    """
    datas = []
    opath = os.getcwd()
    if files == []:
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('_Result.txt', path)
                if a:
                    files.append(path[:-11])
        files.sort()
    #print(files)
    if energy_pattern == 'Energy':
        for file in files:
            #try:
            #print(os.getcwd())
            data = status(file, loc).converged_energy()
            datas.append(data)
            #except:
            #    datas.append(None)
    elif energy_pattern == 'Gibbs':
        for file in files:
            #try:
            data = status(file, loc).gibbs()
            datas.append(data)
            #except:
            #    datas.append(None)
    os.chdir(opath)
    os.chdir(loc)
    Ebar = np.array(datas)
    if absolute == False:
        Ebar = Ebar - datas[0]
    if hartree == False:
        Ebar = Ebar * 627.509
    Ebar.tolist()
    Emax = -99999
    for E in Ebar:
        try:
            if E > Emax:
                Emax = E
        except:
            pass
    Emin = min(Ebar)
    gap = Emax - Emin
    if xx == False:
        if figdata == True:
            if fig_pattern == 'line':
                for i in range(len(Ebar)):
                    if files[i] == '':
                        continue
                    if Emax - Ebar[i] <= 5.0:
                        plt.text(i-0.5, Ebar[i] - 0.05*gap, '{0:.1f}'.format(Ebar[i]))
                    elif Emax - Ebar[i] > 5.0:
                        plt.text(i-0.2, Ebar[i] + 0.02*gap, '{0:.1f}'.format(Ebar[i]))
            elif fig_pattern == 'bar':
                for i in range(len(Ebar)):
                    plt.text(i, Ebar[i], '{0:.1f}'.format(Ebar[i]))
        if fig_pattern == 'line':
            ax = plt.plot(files, Ebar, '*-')
            if energy_pattern == 'Energy' and hartree == True:
                plt.ylabel('∆E(Eh)')
            elif energy_pattern == 'Energy' and hartree == False:
                plt.ylabel('∆E(kcal/mol)')
            elif energy_pattern == 'Gibbs' and hartree == True:
                plt.ylabel('∆G(Eh)')
            elif energy_pattern == 'Gibbs' and hartree == False:
                plt.ylabel('∆G(kcal/mol)')
        elif fig_pattern == 'bar':
            ax=plt.bar(files, Ebar)
            if energy_pattern == 'Energy' and hartree == True:
                plt.ylabel('∆E(Eh)')
            elif energy_pattern == 'Energy' and hartree == False:
                plt.ylabel('∆E(kcal/mol)')
            elif energy_pattern == 'Gibbs' and hartree == True:
                plt.ylabel('∆G(Eh)')
            elif energy_pattern == 'Gibbs' and hartree == False:
                plt.ylabel('∆G(kcal/mol)')
    if savefig == True:
        plt.savefig('{0}.png'.format(figname))
    if xx == False:
        plt.show()
    if savedata == True:
        try:
            f = open('{0}.csv'.format(dataname), 'x')
        except:
            f = open('{0}.csv'.format(dataname), 'w')
        f.write('Step,File,')
        if energy_pattern == 'Energy':
            f.write('∆E')
        elif energy_pattern == 'Gibbs':
            f.write('∆G')
        if hartree == True:
            f.write('(Eh)')
        else:
            f.write('(kcal/mol)')
        f.write('\n')
        for i in range(len(files)):
            f.write('{0},{1},{2}\n'.format(i, files[i], Ebar[i]))
        f.close()
    os.chdir(opath)
    return Ebar.tolist()


from MCPoly.lmpset import mould
import re
import os
import pytest

@pytest.fixture
def atoms():
    return mould('Poly1', loc='./data_lmpset/')

def test_cube(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.cube(6, 5, 6, 5, 3, 5)
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly1_663.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    i7 = 0
    i8 = 0
    i9 = 0
    for line in fl:
        a = re.search('972 atoms', line)
        if a:
            i1 = 1
        b = re.search('1404 angles', line)
        if b:
            i2 = 1
        c = re.search('432 impropers', line)
        if c:
            i3 = 1
        d = re.search('4 atom types', line)
        if d:
            i4 = 1
        e = re.search('4 bond types', line)
        if e:
            i5 = 1
        f = re.search('4 dihedral types', line)
        if f:
            i6 = 1
        g = re.search('0.00000 53.32948 xlo xhi', line)
        if g:
            i7 = 1
        h = re.search('0.00000 54.87382 ylo yhi', line)
        if h:
            i8 = 1
        i = re.search('0.00000 26.46613 zlo zhi', line)
        if i:
            i9 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 * i7 * i8 * i9 != 0

@pytest.fixture
def atoms():
    return mould('Poly1', loc='./data_lmpset/')

def test_brick_xy(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.brick(6, 5, 6, 5, 4, 5, xpattern='y')
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly1_664_brickxy.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    i7 = 0
    i8 = 0
    i9 = 0
    for line in fl:
        a = re.search('1188 atoms', line)
        if a:
            i1 = 1
        b = re.search('1056 bonds', line)
        if b:
            i2 = 1
        c = re.search('1584 dihedrals', line)
        if c:
            i3 = 1
        d = re.search('4 atom types', line)
        if d:
            i4 = 1
        e = re.search('5 angle types', line)
        if e:
            i5 = 1
        f = re.search('1 improper types', line)
        if f:
            i6 = 1
        g = re.search('0.00000 53.32948 xlo xhi', line)
        if g:
            i7 = 1
        h = re.search('0.00000 54.87382 ylo yhi', line)
        if h:
            i8 = 1
        i = re.search('0.00000 36.95484 zlo zhi', line)
        if i:
            i9 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 * i7 * i8 * i9 != 0

@pytest.fixture
def atoms():
    return mould('Poly1',loc='./data_lmpset/')

def test_brick_yz(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.brick(6, 5, 6, 5, 4, 5, ypattern='z')
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly1_664_brickyz.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    i7 = 0
    i8 = 0
    i9 = 0
    for line in fl:
        a = re.search('1134 atoms', line)
        if a:
            i1 = 1
        b = re.search('1008 bonds', line)
        if b:
            i2 = 1
        c = re.search('1512 dihedrals', line)
        if c:
            i3 = 1
        d = re.search('4 atom types', line)
        if d:
            i4 = 1
        e = re.search('5 angle types', line)
        if e:
            i5 = 1
        f = re.search('1 improper types', line)
        if f:
            i6 = 1
        g = re.search('0.00000 53.32948 xlo xhi', line)
        if g:
            i7 = 1
        h = re.search('0.00000 54.87382 ylo yhi', line)
        if h:
            i8 = 1
        i = re.search('0.00000 36.95484 zlo zhi', line)
        if i:
            i9 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 * i7 * i8 * i9 != 0

@pytest.fixture
def atoms():
    return mould('Poly1', loc='./data_lmpset/')

def test_brick_zx(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.brick(6, 5, 6, 5, 4, 5, zpattern='x')
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly1_664_brickzx.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    i7 = 0
    i8 = 0
    i9 = 0
    for line in fl:
        a = re.search('1188 atoms', line)
        if a:
            i1 = 1
        b = re.search('1056 bonds', line)
        if b:
            i2 = 1
        c = re.search('1584 dihedrals', line)
        if c:
            i3 = 1
        d = re.search('4 atom types', line)
        if d:
            i4 = 1
        e = re.search('5 angle types', line)
        if e:
            i5 = 1
        f = re.search('1 improper types', line)
        if f:
            i6 = 1
        g = re.search('0.00000 53.32948 xlo xhi', line)
        if g:
            i7 = 1
        h = re.search('0.00000 54.87382 ylo yhi', line)
        if h:
            i8 = 1
        i = re.search('0.00000 36.95484 zlo zhi', line)
        if i:
            i9 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 * i7 * i8 * i9 != 0

@pytest.fixture
def atoms():
    return mould('Poly2', loc='./data_lmpset/')

def test_DataToXyz(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.DataToXyz()
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly2.xyz', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    for line in fl:
        a = re.match('9', line)
        if a:
            i1 = 1
        b = re.search('C       -6.94821584       1.31130517      -1.06055707', line)
        if b:
            i2 = 1
        c = re.search('H       -7.85312000       0.69751000      -1.04387000', line)
        if c:
            i3 = 1
        d = re.search('O       -4.55939129       1.28068339      -1.05975265', line)
        if d:
            i4 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 != 0

@pytest.fixture
def atoms():
    return mould('Poly1', loc='./data_lmpset/')

def test1_DataToMolTxt(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.DataToMolTxt(savename='Poly1_1')
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly1_1.txt', 'r')
    i1 = 0
    i2 = 0
    i3 = 1
    i4 = 0
    i5 = 0
    i6 = 0
    i7 = 0
    i8 = 0
    i9 = 0
    i10 = 0
    for line in fl:
        a = re.search('9 atoms', line)
        if a:
            i1 = 1
        b = re.search('12 dihedrals', line)
        if b:
            i2 = 1
        c = re.search('5 angle types', line)
        if c:
            i3 = 0
        d = re.search('Coords', line)
        if d:
            i4 = 1
        e = re.search('Types', line)
        if e:
            i5 = 1
        f = re.search('Charges', line)
        if f:
            i6 = 1
        g = re.search('Bonds', line)
        if g:
            i7 = 1
        h = re.search('Angles', line)
        if h:
            i8 = 1
        i = re.search('Dihedrals', line)
        if i:
            i9 = 1
        j = re.search('Impropers', line)
        if j:
            i10 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 * i7 * i8 * i9 * i10 != 0

def test2_DataToMolTxt(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.DataToMolTxt(types={1:11}, savename='Poly1_3')
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly1_3.txt', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    for line in fl:
        a = re.match('2 11', line)
        if a:
            i1 = 1
        b = re.match('5 2', line)
        if b:
            i2 = 1
        c = re.match('9 4', line)
        if c:
            i3 = 1
    fl.close()
    assert i1 * i2 * i3 != 0

@pytest.fixture
def atoms():
    return mould('Poly2', loc='./data_lmpset/')

def test_infchain(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.infchain(3)
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly2_Chain.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    for line in fl:
        a = re.search('7 atoms', line)
        if a:
            i1 = 1
        b = re.search('9 angles', line)
        if b:
            i2 = 1
        c = re.search('2 impropers', line)
        if c:
            i3 = 1
        d = re.search('4 bond types', line)
        if d:
            i4 = 1
        e = re.search('4 dihedral types', line)
        if e:
            i5 = 1
        f = re.search('7 3 5 1', line)
        if f:
            i6 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 != 0

def test_chain(atoms):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms.chain(3, 5)
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly2_5x.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    for line in fl:
        a = re.search('37 atoms', line)
        if a:
            i1 = 1
        b = re.search('36 bonds', line)
        if b:
            i2 = 1
        c = re.search('49 angles', line)
        if c:
            i3 = 1
        d = re.search('5 angle types', line)
        if d:
            i4 = 1
        e = re.search('4 dihedral types', line)
        if e:
            i5 = 1
        f = re.match('12 2 14 10', line)
        if f:
            i6 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 != 0

@pytest.fixture
def atoms3():
    return mould('Poly3', loc='./data_lmpset/')

def test_rebuild(atoms3):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests')
    atoms3.rebuild('Poly3_re', x=[-50, 50], y=[-50, 50], z=[-50, 50],\
                   subloc='./data_lmpset/')
    os.chdir(opath)
    fl = open('./MCPoly/tests/data_lmpset/Poly3.data', 'r')
    i1 = 0
    i2 = 0
    i3 = 0
    i4 = 0
    i5 = 0
    i6 = 0
    for line in fl:
        a = re.search('-50.00000 50.00000 xlo xhi', line)
        if a:
            i1 = 1
        b = re.search('-50.00000 50.00000 ylo yhi', line)
        if b:
            i2 = 1
        c = re.search('-50.00000 50.00000 zlo zhi', line)
        if c:
            i3 = 1
        d = re.search('2 1 2 0.7694 9.97863 4.59047 2.66070', line)
        if d:
            i4 = 1
        e = re.search('10 1 7 0.1191 11.55624 6.78551 3.42033', line)
        if e:
            i5 = 1
        f = re.search('73 1 8 -0.4064 19.29389 5.70762 2.26849', line)
        if f:
            i6 = 1
            break
    fl.close()
    assert i1 * i2 * i3 * i4 * i5 * i6 != 0


import re
import os
import sys
import py3Dmol
import matplotlib.pyplot as plt
from ipywidgets import interact
import ipywidgets as iw
from rdkit import Chem
from rdkit.Chem.Draw import IPythonConsole
from rdkit.Chem import rdDetermineBonds

def status_judge(s1, s2, converge):
    if s1 == 0 or s1 == 4 or s1 == 41:
        if s1 == 0:
            print('The Optimization is processing.')
        elif s1 == 4 and s2 == 0:
            print('The Optimization is aborted.')
        elif s1 == 41:
            print('The Optimization is aborted.\n\
                The structure includes 180 or 0 degrees angles.')
        print('Last time:')
        print('Energy change: {0}'.format(converge[-5]))
        print('RMS gradient: {0}             MAX gradient: {0}'.format(
            converge[-4], converge[-3]))
        print('RMS step: {0}                 MAX step: {0}'.format(
            converge[-2], converge[-1]))
    if s1 != 0 and s2<0:
        print('The Optimization was finished.')
        if s1 == 4 or s1 >= 40:
            print('The frequency calculation is aborted.')
            if s1 == 41:
                print('There are some 0 or 180 degrees angles.\n\
                Try to rebuild the system.')
            return None
        if s2 == -1:
            print('The CP-SCF equations are forming.')
        elif s2 == -2:
            print('The CP-SCF equations are solving.')
        elif s2 == -3:
            print('The Thermodynamics Calculation was processing.')
    if s2 == 1:
        print('The Thermodynamics Calculation was finished.')
    if s2 == 9 and s1 == 6:
        print('The Thermodynamics Calculation was finished, but\
            the optimization did not converge and only\
            reached the maximum number of optimization cycles.')
        
def statusfig(energy, choose, num=-1):
    t = range(len(energy))
    if choose == 0:
        plt.plot(t, energy, 'x-')
    else:
        plt.plot(t[-choose:], energy[-choose:], 'x-')
    if num >= 0:
        #plt.plot(num, energy[num], 'r', markersize=12)
        plt.scatter(num, energy[num], s=40, c='r')
    plt.show()

def higeo(file, num):
    energy = status(file, figureonly=True)
    statusfig(energy, 0, num)
    
def cleanup_qm9_xyz(fname):
    ind = open(fname).readlines()
    nAts = int(ind[0])
    # There are two smiles in the data: the one from GDB and the one assigned from the
    # 3D coordinates in the QM9 paper using OpenBabel (I think).
    gdb_smi, relax_smi = ind[-2].split()[:2]
    ind[1] = '\n'
    ind = ind[:nAts+2]
    for i in range(2, nAts+2):
        l = ind[i]
        l = l.split('\t')
        l.pop(-1)
        ind[i] = '\t'.join(l)+'\n'
    ind = ''.join(ind)
    return ind, gdb_smi, relax_smi

def draw_with_spheres(mol, width, height):
    v = py3Dmol.view(width=width, height=height)
    IPythonConsole.addMolToView(mol, v)
    v.zoomTo()
    v.setStyle({'sphere':{'radius':0.4}, 'stick':{'radius':0.1}});
    v.show()
    return v

def multicreatemol(filename, turn, width, height, MD=False):
    if MD == True:
        try:
            ind = open('{0}_dump.xyz'.format(filename), 'r+')
        except:
            raise FileNotFoundError('Perhaps your _dump.xyz file has some errors.\
            Please check the location of\
            your _dump.xyz file and then modify your file.')
    else:
        try:
            ind = open('{0}_trj.xyz'.format(filename), 'r+')
        except:
            raise FileNotFoundError('Perhaps your _trj.xyz file has some errors.\
            Please check the location of\
            your _trj.xyz file and then modify your file.')
    mains = ind.readlines()
    num = eval(mains[0][:-1])
    term = len(mains)/(num+2)
    main = ''
    partmain = mains[turn*(num+2):(turn+1)*(num+2)]
    for i, line in enumerate(partmain):
        if i == 1:
            main = main + '\n'
        else:
            main = main + partmain[i]
    #print(main)
    raw_mol = Chem.MolFromXYZBlock(main)
    conn_mol = Chem.Mol(raw_mol)
    rdDetermineBonds.DetermineConnectivity(conn_mol)
    v = draw_with_spheres(conn_mol, width, height)
    return v

def normalstatus(loc, file, choose=0, figureonly=False,\
                 statusonly=False, MD=False):
    path = os.getcwd()
    os.chdir(loc)
    file = file + '.out'
    if figureonly == False and statusonly == False:
        print(file)
    s1 = 0
    s2 = 0
    term = 0
    f = open(file, 'r')
    x = 0
    energy = []
    converge = []
    if MD == False:
        for line in f:
            a = re.search('FINAL SINGLE POINT ENERGY', line)
            if a:
                a2 = re.search(r'-[0-9]+\.[0-9]+', line)
                energy.append(eval(a2.group(0)))
            x1 = re.search('OPTIMIZATION RUN DONE', line)
            if x1:
                s1 = 1
            if s1 == 1:
                in1 = re.search('Forming right-hand sides of CP-SCF equations', line)
                if in1:
                    s2 = -1
                in2 = re.search('Solving the CP-SCF equations', line)
                if in2:
                    s2 = -2
                in3 = re.search('VIBRATIONAL FREQUENCIES', line)
                if in3:
                    s2 = -3
            x2 = re.search('ORCA TERMINATED NORMALLY', line)
            if x2:
                if s1 == 6:
                    s2 = 9
                else:
                    s2 = 0
            t = re.search('GEOMETRY OPTIMIZATION CYCLE', line)
            if t:
                t2 = re.search(r'[0-9]+', line)
                term = t2.group(0)
            c = re.search('-|Geometry convergence|-', line)
            if c:
                x = 1
            if x == 2:
                yes = re.search('YES', line)
                no = re.search('NO', line)
                if yes:
                    converge.append('YES')
                if no:
                    converge.append('NO')
                u = re.search('-------------------------', line)
                if u:
                    x = 0
            if x == 1:
                u = re.search('-------------------------', line)
                if u:
                    x = 2
            x3 = re.search('aborting the run', line)
            if x3:
                if s1 == 41:
                    break
                else:
                    s1 = 4
            x3 = re.search('ANGLE IS APPROACHING 180', line)
            if x3:
                s1 = 41
            if s1 == 41:
                x3 = re.search('GEOMETRY OPTIMIZATION CYCLE', line)
                if x3:
                    s1 = 0
            x4 = re.search('did not converge but reached', line)
            if x4:
                s1 = 6
    else:
        s1 = 0
        s2 = 0
        ii = 0
        gap = 0
        for line in f:
            x2 = re.search('ERROR', line)
            if x2:
                s1 = 4
                break
            x3 = re.search('ORCA TERMINATED NORMALLY', line)
            if x3:
                s1 = 1
        for line in f:
            x0 = re.search('Dump Position Stride [0-9]+', line)
            if x0:
                y0 = re.search(r'[0-9]+', x0.group(0))
                gap = eval(y0.group(0))
            x1 = re.findall('\-\|', line)
            if len(x1) > 3:
                if ii == 0:
                    ii = 1
                else:
                    break
            elif ii == 1:
                alls = re.findall(r'\-*[0-9]+\.*[0-9]*', line)
                if alls[0] == '0':
                    energy.append(eval(alls[-1]))
                else:
                    if eval(alls[0]) % gap == 0:
                        energy.append(eval(alls[-3]))
            
    f.close()
    if statusonly == True:
        if s1 == 1 and s2 == 0:
            os.chdir(path)
            return 2
        else:
            os.chdir(path)
            return s1
    if figureonly == False and MD == False:
        print('{0} turns have been calculated.'.format(term))
        status_judge(s1, s2, converge)
        statusfig(energy, choose)
    elif figureonly == False and MD == True:
        print('{0} turns have been calculated.'.format(len(energy)))
        statusfig(energy, choose)
    os.chdir(path)
    return energy

def thermo(loc, file, keyword):
    path = os.getcwd()
    os.chdir(loc)
    f = open(file+'.out', 'r')
    for line in f:
        if keyword == 'Gibbs':
            a = re.search('Final Gibbs free energy', line)
            if a:
                b = re.search(r'-[0-9]+\.[0-9]+', line)
                os.chdir(path)
                return '{0:.6f}'.format(eval(b.group(0)))
        if keyword == 'Enthalpy':
            a = re.search('Total enthalpy', line)
            if a:
                b = re.search(r'-[0-9]+\.[0-9]+', line)
                os.chdir(path)
                return '{0:.6f}'.format(eval(b.group(0)))
        if keyword == 'Entropy':
            a = re.search('Total entropy correction', line)
            if a:
                b = re.findall(r'-[0-9]+\.[0-9]+', line)
                os.chdir(path)
                return '{0:.6f}'.format(eval(b[0]))
    os.chdir(path)
    raise Exception("'{0}.out' may not have keyword freq, or it's aborted.\
        Please check your input file and recalculate it.".format(file))

def charge(loc, file, num, keyword):
    f = open(loc+file+'.out', 'r')
    i = 0
    ac = [None] * 200
    for line in f:
        if keyword == 'Mulliken':
            d = re.search('Sum of atomic charges', line)
            if d:
                i = 0
            a = re.search('MULLIKEN ATOMIC CHARGES', line)
            if a:
                i = 1
            if i == 1:
                b = re.search(r'-?[0-9]+\.[0-9]+', line)
                if b:
                    n = re.findall(r'[0-9]+', line)
                    ac[eval(n[0])] = eval(b.group(0))
        elif keyword == 'Loewdin':
            a = re.search('LOEWDIN ATOMIC CHARGES', line)
            if a:
                i = 1
            if i == 1:
                b = re.search(r'-?[0-9]+\.[0-9]+', line)
                if b:
                    n = re.findall(r'[0-9]+', line)
                    ac[eval(n[0])] = eval(b.group(0))
            d = re.search('LOEWDIN REDUCED ORBITAL CHARGES', line)
            if d:
                i = 0
    return ac[num]

def AtomsAll(loc, file):
    path = os.getcwd()
    os.chdir(loc)
    f = open(file+'.out', 'r')
    i = 0
    n = 0
    atoms = [['NOT', '99999', '99999', '99999']] * 200
    elements = ['NOT'] * 200
    xyzs = [['99999', '99999', '99999']] * 200
    for line in f:
        d = re.search('CARTESIAN COORDINATES \(A\.U\.\)', line)
        if d:
            i = 0
            n = 0
        if i == 1:
            b = re.search(r'[A-Z][a-z]*', line)
            if b:
                xyz = re.findall(r'-?[0-9]+\.[0-9]+', line)
                try:
                    elements[n] = b.group(0)
                except:
                    elements.append(b.group(0))
                try:
                    xyzs[n] = xyz
                except:
                    xyzs.append(xyz)
                try:
                    atoms[n] = [b.group(0), eval(xyz[0]), eval(xyz[1]), eval(xyz[2])]
                except:
                    atoms.append([b.group(0), eval(xyz[0]), eval(xyz[1]), eval(xyz[2])])
                n = n + 1
        a = re.search('CARTESIAN COORDINATES \(ANGSTROEM\)', line)
        if a:
            i = 1
    while 1:
        try:
            atoms.remove(['NOT', '99999', '99999', '99999'])
            elements.remove('NOT')
            xyzs.remove(['99999', '99999', '99999'])
        except:
            break
    f.close()
    os.chdir(path)
    return atoms

def AtomsAllXyz(loc, file, error_from_no_out = False):
    if error_from_no_out == False:
        path = os.getcwd()
        os.chdir(loc)
    else:
        pass
    print(os.getcwd())
    f = open(file+'.xyz', 'r')
    atoms = [['NOT', '99999', '99999', '99999']] * 200
    elements = ['NOT'] * 200
    xyzs = [['99999', '99999', '99999']] * 200
    i = 0
    for line in f:
        b = re.findall(r'-?[0-9]+\.[0-9]+', line)
        if len(b) == 3:
            ele = re.search(r'[A-Z][a-z]?', line)
            try:
                atoms[i] = [ele.group(0), eval(b[0]), eval(b[1]), eval(b[2])]
            except:
                atoms.append([ele.group(0), eval(b[0]), eval(b[1]), eval(b[2])])
        i = i + 1
    while 1:
        try:
            atoms.remove(['NOT', '99999', '99999', '99999'])
            elements.remove('NOT')
            xyzs.remove(['99999', '99999', '99999'])
        except:
            break
    f.close()
    if error_from_no_out == False:
        os.chdir(path)
    else:
        pass
    return atoms

class status:
    """
    A method to see the current process of the ORCA optimisation, including convergence situation and relevant energy chart.
    status(file, loc='./')
    file: File Name.
    loc: File Location. The default is your current location.
    You can get the further information by .status and .figures.
    """

    def __init__(self, file, loc='./'):
        self.file = file
        self.loc = loc
    
    def status(self, choose=0, figureonly=False, statusonly=False, MD=False):
        """
    A method to see the current process of the ORCA optimisation, including convergence situation and relevant energy chart.
    status(choose=0, figureonly=False, statusonly=False, last=-1)
    choose: See the latest energy process. e.g. When choose=5, it will show the last five energy data on the chart.
    figureonly: If 'figureonly' is True, the code will show no status information. The default is False.
    statusonly: If 'statusonly' is True, the code will output the current process status only, not energy. 
                in output s, 0 means the system is optimising, 1 means the system completes the optimisation and starts frequency calculation, 2 means the process is completed, 4 means the process is aborted.
                The default is False.
                TIPS: Kill ORCA process with problems other than ORCA will get the output of 0.
    Normally, the output of this function is the energy change in optimisation, unless statusonly=True.
    Example 1:
        Input:
            from MCPoly import status
            a = status('Et', '/Molecule').status()
        
        Output:
            Et.out
            8 turns have been calculated.
            The Thermodynamics Calculation was finished.
            <chart from matplotlib>
            
    Example 2:
        Input:
            from MCPoly import status
            a = status('Molecule 1', '/Molecule').status(figureonly=True)
            print(a)
        
        Output:
            [-1729.508137253801, -1729.505855679262, -1729.501743481437, -1729.499301095603, -1729.499960765849, 
             -1729.499875258488, -1729.500097690162, -1729.500368245231, -1729.500825118936, -1729.501319993802, 
             -1729.503640287060, -1729.505369082598, -1729.507234351098, -1729.508676755388, ...]
    
    Example 3:
        Input:
            from MCPoly import status
            a = status('SI5_0.5', '../Sulphur').status(figureonly=True, statusonly=True)
            print(a)
        
        Output:
            2
        """

        return normalstatus(self.loc, self.file, choose, figureonly, statusonly, MD)
    
    def energy(self):
        """
    A method to see the final(or latest) Energy of a system.
    energy()
        """
        E = normalstatus(self.loc, self.file, figureonly=True)
        return eval('{0:.6f}'.format(E[-1]))

    def converged_energy(self, unit='Eh'):
        """
    A method to see the Converged Energy of a system.
    converged_energy(unit)
    unit: Energy unit, including 'Eh' and 'kcal/mol'. The default is 'Eh' (Hartree).
        """
        E = normalstatus(self.loc, self.file, figureonly=True)
        state = normalstatus(self.loc, self.file, figureonly=True, statusonly=True)
        if state != 2:
            raise Exception("This optimisation of '{0}.out' is failed. No converged energy.".format(self.file))
        else:
            x = eval('{0:.6f}'.format(E[-1]))
            if unit == 'Eh':
                return x
            elif unit == 'kcal/mol':
                return x * 627.509
    
    def gibbs(self, unit='Eh'):
        """
    A method to see the Gibbs Free Energy of a system.
    gibbs(unit)
    unit: Energy unit, including 'Eh' and 'kcal/mol'. The default is 'Eh' (Hartree).
    TIPS: Make sure the optimisation includes key word 'freq', which means frequency calculation.
        """
        x = eval(thermo(self.loc, self.file, keyword='Gibbs'))
        if unit == 'Eh':
            return x
        elif unit == 'kcal/mol':
            return x * 627.509
    
    def enthalpy(self, unit='Eh'):
        """
    A method to see the total enthalpy of a system.
    enthalpy(unit)
    unit: Energy unit, including 'Eh' and 'kcal/mol'. The default is 'Eh' (Hartree).
    TIPS: Make sure the optimisation includes key word 'freq', which means frequency calculation.
        """
        x = eval(thermo(self.loc, self.file, keyword='Enthalpy'))
        if unit == 'Eh':
            return x
        elif unit == 'kcal/mol':
            return x * 627.509
    
    def entropy_correction(self, unit='Eh'):
        """
    A method to see the total entropy correction of a system.
    enthalpy(unit)
    unit: Energy unit, including 'Eh' and 'kcal/mol'. The default is 'Eh' (Hartree).
    TIPS: Make sure the optimisation includes key word 'freq', which means frequency calculation.
        """
        x = eval(thermo(self.loc, self.file, keyword='Entropy'))
        if unit == 'Eh':
            return x
        elif unit == 'kcal/mol':
            return x * 627.509
    
    def mulliken_charge(self, num):
        """
    A method to see the Mulliken Charge of an atom.
    mulliken_charge(num)
    num: Atom number.
    TIPS: The atom starts from 0. Don't surpass the maximum, or it will cause error.
        """
        x = charge(self.loc, self.file, num, keyword='Mulliken')
        return x
    
    def loewdin_charge(self, num):
        """
    A method to see the Loewdin Charge of an atom.
    loewdin_charge(num)
    num: Atom number.
    TIPS: The atom starts from 0. Don't surpass the maximum, or it will cause error.
        """
        x=charge(self.loc, self.file, num, keyword='Loewdin')
        return x

    def atom(self):
        """
    A method to find out the atom information of the system.
    atom()
    
    TIPS: In the following list, a piece of atom information is like that:
        ['C', 4.11199532065866, 1.93907233706568, -1.49149364116961]
        The first one is the element type, the following three numbers are location in x, y, z-axis.
        """
        try:
            return AtomsAll(self.loc, self.file)
        except:
            return AtomsAllXyz(self.loc, self.file, error_from_no_out = True)
    
    def mass(self):
        """
    A method to find out the mass of all particles in system.
    mass()
        """
        M = 0
        try:
            x = AtomsAll(self.loc, self.file)
        except:
            x = AtomsAllXyz(self.loc, self.file, error_from_no_out = True)
        for i in x:
            if i[0] == 'H':
                M = M + 1.008
            elif i[0] == 'C':
                M = M + 12.011
            elif i[0] == 'O':
                M = M + 15.999
            elif i[0] == 'Li':
                M = M + 6.94
            elif i[0] == 'Be':
                M = M + 9.0122
            elif i[0] == 'B':
                M = M + 10.81
            elif i[0] == 'N':
                M = M + 14.007
            elif i[0] == 'F':
                M = M + 18.998
            elif i[0] == 'Na':
                M = M + 22.990
            elif i[0] == 'Mg':
                M = M + 24.305
            elif i[0] == 'Al':
                M = M + 26.982
            elif i[0] == 'Si':
                M = M + 28.085
            elif i[0] == 'P':
                M = M + 30.974
            elif i[0] == 'S':
                M = M + 32.06
            elif i[0] == 'Cl':
                M = M + 35.45
            elif i[0] == 'Br':
                M = M + 79.904
            elif i[0] == 'I':
                M = M + 126.90
            elif i[0] == 'K':
                M = M + 39.098
            elif i[0] == 'Ca':
                M = M + 40.078
            elif i[0] == 'Ti':
                M = M + 47.867
            elif i[0] == 'Cr':
                M = M + 51.996
            elif i[0] == 'Mn':
                M = M + 54.938
            elif i[0] == 'Fe':
                M = M + 55.845
            elif i[0] == 'Co':
                M = M + 58.933
            elif i[0] == 'Ni':
                M = M + 58.693
            elif i[0] == 'Cu':
                M = M + 63.546
            elif i[0] == 'Zn':
                M = M + 65.38
            elif i[0] == 'Ru':
                M = M + 101.07
            elif i[0] == 'Rh':
                M = M + 102.91
            elif i[0] == 'Ag':
                M = M + 107.87
            elif i[0] == 'Ir':
                M = M + 192.22
            elif i[0] == 'Pt':
                M = M + 195.08
            elif i[0] == 'Au':
                M = M + 196.97
            elif i[0] == 'Hg':
                M = M + 200.59
        return M
    
    def atom_num(self):
        """
    A method to find out the atom number of the system.
    atom_num()
        """
        return len(AtomsAll(self.loc, self.file))
    
    def steps(self, MD=False):
        """
    A method to know the total steps of the ORCA runs.
    TIPS: Make sure your _dump.xyz file is in the document with .out file, or there will be FileNotFoundError!!!
    
    steps(MD=False)
    MD: See the single status of Molecular Dynamics (MD). The default is False.
        """
        figure = normalstatus(self.loc, self.file, figureonly=True, MD=MD)
        return len(figure) - 1
    
    def figure(self, num=0, width=300, height=300, MD=False, save=''):
        """
    A method to see the current geometry structure of the ORCA optimisation, powered by py3Dmol and rdkit.
    TIPS: Make sure your _trj.xyz file is in the document with .out file, or there will be NoFileFoundError!!!
    
    figure(num=0, width=300, height=300, MD=False, save='')
    num: The step of your convergence.
    width, height: The size of your 3D geometry molecule strcuture. Default: 300x300.
    MD: See the single status of Molecular Dynamics (MD). The default is False.
    save: The name of the single status XYZ file you want to save. The default is not saving the single status.
        """
            
        figure = normalstatus(self.loc, self.file, figureonly=True, MD=MD)
        file = self.file
        try:
            path = os.getcwd()
            os.chdir(self.loc)
            multicreatemol(file, num, width, height)
            os.chdir(path)
        except:
            raise ValueError("The index 'num' is out of range.")
        
    def figuretraj(self, num=0, width=300, height=300):
        """
    A method to see the current geometry structure and optimization trajectory of the ORCA optimisation, powered by py3Dmol and ipywidgets package.
    TIPS: Make sure your _trj.xyz file is in the document with .out file, or there will be FileNotFoundError!!!
    
    figurestatus(num=0, width=300, height=300)
    num: The step of your convergence. The default is the origin structure.
    width, height: The size of your 3D geometry molecule strcuture. Default: 300x300.
    After forming the 3D geometry molecule strcuture, you can scroll to see other structures of relevant molecules.
        """
            
        figure = normalstatus(self.loc, self.file, figureonly=True)
        file = self.file
        def turn(num):
            try:
                path = os.getcwd()
                os.chdir(self.loc)
                multicreatemol(file, num, width, height)
                os.chdir(path)
            except:
                if num == len(figure) - 1:
                    print("The last step hasn't been optimised yet.")
        interact(turn, num=iw.IntSlider(min=0, max=len(figure)-1, step=1, value=num))
    
    def figuremd(self, num=0, width=300, height=300):
        """
    A method to see the current geometry structure and optimization trajectory of the ORCA Molecular Dynamics, powered by py3Dmol and ipywidgets package.
    TIPS: Make sure your _dump.xyz file is in the document with .out file, or there will be FileNotFoundError!!!
    
    figuremd(num=0, width=300, height=300)
    num: The step of your convergence. The default is the origin structure.
    width, height: The size of your 3D geometry molecule strcuture. Default: 300x300.
    After forming the 3D geometry molecule strcuture, you can scroll to see other structures of relevant molecules.
        """
            
        figure = normalstatus(self.loc, self.file, figureonly=True, MD=True)
        file = self.file
        def turn(num):
            try:
                path = os.getcwd()
                os.chdir(self.loc)
                multicreatemol(file, num, width, height, MD=True)
                os.chdir(path)
            except:
                if num == len(figure) - 1:
                    print("The last step hasn't been optimised yet.")
        interact(turn, num=iw.IntSlider(min=0, max=len(figure), step=1, value=num))
    
    def figurecharge(self, width=300, height=300):
        """
    A method to see the Mulliken Charge of each atom base on 3D geometry structure of the system.
    TIPS: Make sure your .xyz file is in the document with .out file, or there will be FileNotFoundError!!!
    
    width, height: The size of your 3D geometry molecule strcuture. Default: 300x300.
    After forming the 3D geometry molecule strcuture, you can scroll to see the charge of the relevant atom.
        """
        output = iw.Output()
        mydir = os.path.dirname( __file__ )
        orcadir = os.path.join(mydir, '..', 'orcaset')
        sys.path.append(orcadir)
        from view3dchoose import view3dchoose
        
        chargetype = iw.Dropdown(description='Atomic Charge Type',\
                                 options=['Mulliken', 'Loewdin'])
        num = iw.IntSlider(value=0, min=0,\
                           max=len(AtomsAll(self.loc, self.file))-1)
        
        def GUI(chargetype, num):
            output.clear_output()
            with output:
                alls = AtomsAll(self.loc, self.file)
                view3dchoose(self.file, self.loc,\
                             choose=[num], width=width, height=height)
                print('Atom: {0}, Element: {1}, Charge: {2}'.format(num,\
                                                                    alls[num][0], charge(self.loc, self.file, num, chargetype))) 
            
        geoout = iw.interactive_output(GUI, {'chargetype': chargetype, 'num': num})
        display(iw.VBox([chargetype, num, output]))


import os
import re
import warnings
import sys
import time as t
import pandas as pd
import seaborn as sns
import ipywidgets as widgets
import matplotlib.pyplot as plt
from IPython.display import display
from .YModulus import YModulus
from .single import single

mydir = os.path.dirname( __file__ )
viewdir = os.path.join(mydir, '..', 'orcaset')
sys.path.append(viewdir)
from view3dchoose import view3dchoose

mydir = os.path.dirname( __file__ )
viewdir = os.path.join(mydir, '..', 'orcaset')
sys.path.append(viewdir)
from XyzToInp import XyzToInp

mydir = os.path.dirname( __file__ )
statusdir = os.path.join(mydir, '..', 'status')
sys.path.append(statusdir)
from status import status

def gui():
    """
        The method to use GUI to assemble mechanical datas and get the multiple stress-strain curve on a chart in handy!
        You will know how to do as long as you get into the GUI platform.

        TIPS: 1. Make sure all your created .xyz files from orcaset.ssorca or orcaset.ssgui is on the direct folder, or the programme will get error.
              2. You'd better have relevant .inp or .out file on the direct folder to automatically set the site of external force. Though GUI has manual auxiliary, it will cost you much more time.
              3. The visualisation will also risk the flexibility of the task, to make you programme more flexible, please directly use the code.
    """
    def multiple_gui(polymers, loc='./', savedata=False,\
                     savefig=False, pngname='', csvname=''):
        opath=os.getcwd()
        os.chdir(loc)
        try:
            f = open('ot.txt', 'x')
        except:
            f = open('ot.txt', 'w')
        
        name = []
        i = 0
        
        site = []
        
        for polymer in polymers:
            polymer = polymer + '_Result.txt'
            try:
                datum = pd.read_csv(polymer)
            except:
                warnings.warn("'{0}' is not found.".format(polymer))
                continue
            datum = datum[:].sort_values(by=['Strain Length(%)'])
            for num in datum['Stress Force(nN)'][:-1]:
                if num not in site:
                    site.append(num)
        site.sort()
        f.write('Stress Force(nN)\n')
        for num in site:
            f.write('{0:.3f}\n'.format(num))
        f.close()
        ot = pd.read_csv('ot.txt')
        for polymer in polymers:
            polymer = polymer + '_Result.txt'
            try:
                datum = pd.read_csv(polymer)
            except:
                continue
            try:
                datum = datum[:].sort_values(by=['Strain Length(%)'])
            except:
                continue
            ax = plt.plot(datum['Strain Length(%)'][:-1],\
                          datum['Stress Force(nN)'][:-1], '*-')
            datum = datum.rename(
                columns={'Strain Length(%)':'Strain Length ({0})'.format(polymer[:-11])})
            ot = pd.merge_ordered(ot, datum[:-1], fill_method="ffill",\
                                  left_by="Stress Force(nN)")
            name.append(polymer[:-11])
            i = i + 1
    
        ot = ot.set_index('Stress Force(nN)')
        if savedata == True or savefig == True:
            if savedata == True:
                ot.to_csv('{0}.csv'.format(csvname))
        else:
            output3.clear_output()
            with output3:
                display(ot)
                display(widgets.HBox([widgets.Label('CSV File Name:'), name2]))
                display(save2)
        
        plt.xlabel('Strain Length(%)')
        plt.ylabel('Stress Force(nN)')
        plt.legend(name, bbox_to_anchor=[1.1, 0.9])
        _ = plt.title('Stress-Strain Curve')
        if savedata == True or savefig == True:
            if savefig == True:
                plt.savefig('{0}.png'.format(pngname), bbox_inches='tight')
        else:
            with output2:
                plt.show()
                display(widgets.HBox([widgets.Label('PNG File Name:'), name1]))
                display(save1)
    
        os.system('rm ot.txt')
        os.chdir(opath)
    
    print('TIPS: Multiple values can be selected with '
          + 'Shift and/or Ctrl (or Command) pressed '
          + 'and mouse clicks or arrow keys.')
    
    output2 = widgets.Output()
    output3 = widgets.Output()
    judgeoutput = widgets.Output()
    
    def geoshow(aim1, aim2, geoi, files):
        output2.clear_output()
        with output2:
            try:
                print("'{0}.xyz' Geometry Structure:".format(files[geoi]))
            except:
                return None
            aim = [aim1, aim2]
            print('Atom1: {0}      Atom2: {1}'.format(aim1, aim2))
            try:
                view3dchoose(files[geoi]+'_0.000', loc.value,\
                             choose=aim, width=450, height=400)
            except:
                view3dchoose(files[geoi]+'_0', loc.value, choose=aim, width=450, height=400)
    
    def changecombo(loc):
        options2 = []
        for path in os.listdir(loc):
            if os.path.isfile(os.path.join(loc, path)):
                a = re.search('.out', path)
                if a:
                    options2.append(path[:-4])
        options2.sort()
        files.options = options2
    
    def filejudge(files, loc):
        judgeoutput.clear_output()
        output2.clear_output()
        output3.clear_output()
        opath = os.getcwd()
        os.chdir(loc)
        #geoi = widgets.IntText(value=0)
        t0 = 0
        try:
            geoi.value = 0
            for file in files:
                t0 = 0
                for path in os.listdir(loc):
                    if os.path.isfile(os.path.join(loc, path)):
                        a = re.match('{0}_Result.txt'.format(file), path)
                        if a:
                            output2.clear_output()
                            geoi.value = geoi.value + 1
                            t0 = 1
                            break
                if t0 == 0:
                    output2.clear_output()
                    single(file, loc=loc).autocurve(savefig=False)
                    geoi.value = geoi.value + 1
                #single(file, loc=loc).autocurve(savefig=False)
                #geoi.value = geoi.value + 1
        except:
            xtitle = widgets.Label('Extra Information about {0}:'.format(
                files[geoi.value]), style=dict(font_weight='bold'))
            #aim1 = widgets.BoundedIntText(description='Atom 1:', value=0, min=0, layout=widgets.Layout(width='48%'))
            #aim2 = widgets.BoundedIntText(description='Atom 2:', value=0, min=0, layout=widgets.Layout(width='48%'))
            aim1.value = 0
            aim2.value = 0
            OK = widgets.Button(description='OK')
            with judgeoutput:
                display(widgets.VBox([xtitle, widgets.HBox([aim1, aim2]), OK]))
            with output2:
                #print("'{0}' Geometry Structure:".format(file))
                #geoout = widgets.interactive_output(geoshow, {'aim1': aim1, 'aim2': aim2, 'geoi': geoi})
                OK.on_click(OKss)
        os.chdir(opath)
    
    def OKss(button):
        opath = os.getcwd()
        os.chdir(loc.value)
        #with output3:
        #    print(geoi.value, aim1.value, aim2.value)
        single(files.value[geoi.value], loc=loc.value).curve(
            aim1.value, aim2.value, savefig=False)
        judgeoutput.clear_output()
        output2.clear_output()
        geoi.value = geoi.value + 1
        try:
            if geoi.value >= len(files.value):
                return None
            for i in range(geoi.value, len(files.value)):
                for path in os.listdir(loc.value):
                    if os.path.isfile(os.path.join(loc.value, path)):
                        a = re.match(files.value[geoi.value]+'_Result.rxt', path)
                        if a:
                            geoi.value = geoi.value + 1
                            return None
                single(files.value[i], loc=loc.value).autocurve(savefig=False)
                geoi.value = geoi.value + 1
        except:
            xtitle = widgets.Label(
                'Extra Information about {0}:'.format(files.value[geoi.value]),\
                style=dict(font_weight='bold'))
            #aim1 = widgets.BoundedIntText(description='Atom 1:',\
            #                              value=0, min=0, layout=widgets.Layout(width='48%'))
            #aim2 = widgets.BoundedIntText(description='Atom 2:',\
            #                              value=0, min=0, layout=widgets.Layout(width='48%'))
            OK = widgets.Button(description='OK')
            with judgeoutput:
                display(widgets.VBox([xtitle, widgets.HBox([aim1, aim2]), OK]))
            with output2:
                #print("'{0}.xyz' Geometry Structure:".format(files.value[geoi.value]))
                #geoout = widgets.interactive_output(geoshow,\
                #                                    {'aim1': aim1, 'aim2': aim2, 'geoi': geoi})
                #display(geoout)
                OK.on_click(OKss)
        os.chdir(opath)
    
    def intro_result(button):
        for file in files.value:
            t0 = 0
            try:
                t0 = 0
                for path in os.listdir(loc.value):
                    if os.path.isfile(os.path.join(loc.value, path)):
                        a = re.match(file+'_Result.rxt', path)
                        if a:
                            t0 = 1
                            break
                if t0 == 0:
                    single(file, loc=loc.value).autocurve(savefig=False)
            except:
                pass
    #            xtitle = widgets.Label('Extra Information about {0}:'.format(file), style=dict(font_weight='bold'))
    #            aim1 = widgets.BoundedIntText(description='Atom 1:', value=0, min=0, layout=widgets.Layout(width='48%'))
    #            aim2 = widgets.BoundedIntText(description='Atom 2:', value=0, min=0, layout=widgets.Layout(width='48%'))
    #            OK = widgets.Button(description='OK')
    #            with output:
    #                display(widgets.VBox[xtitle, widgets.HBox([aim1, aim2]), OK])
    #            geoout = widgets.interactive_output(geoshow, {'aim1': aim1, 'aim2': aim2})
    #            with outout2:
    #                display(geoout)
    #            OK.on_click(OKss)
    #            t.sleep(15)
        multiple_gui(files.value, loc=loc.value)
        
    def pngsave(button):
        multiple_gui(files.value, loc=loc.value, savefig=True, pngname=name1.value)
    
    def csvsave(button):
        multiple_gui(files.value, loc=loc.value, savedata=True, csvname=name1.value)
    
    options = []
    chooses = []
    title1 = widgets.Label('File Settings:', style=dict(font_weight='bold'))
    loc = widgets.Text(description='Location:', value='./')
    for path in os.listdir(loc.value):
        if os.path.isfile(os.path.join(loc.value, path)):
            a = re.search('.xyz', path)
            if a:
                b = re.search('_', path)
                if b:
                    c = re.search('.', path[:-4])
                    if c:
                        fragment = re.split('_', path)
                        if fragment[0] not in options:
                            options.append(fragment[0])
                        if fragment[1][:-4] not in chooses:
                            chooses.append(fragment[1][:-4])
    options.sort()
    files = widgets.SelectMultiple(options=options, description='Files Series:')
    #ym = widgets.Button(description="Young's Modulus", layout=widgets.Layout(width='100%'))
    resultshow = widgets.Button(description='Show Chart and table',\
                                layout=widgets.Layout(width='100%'),\
                                style=dict(font_weight='bold', text_decoration='underline'))
    save1 = widgets.Button(description='Save', layout=widgets.Layout(width='100%'))
    save2 = widgets.Button(description='Save', layout=widgets.Layout(width='100%'))
    name1 = widgets.Text(value='Result', layout=widgets.Layout(width='60%'))
    name2 = widgets.Text(value='Result', layout=widgets.Layout(width='60%'))
    aim1 = widgets.BoundedIntText(description='Atom 1:', value=0,\
                                  min=0, layout=widgets.Layout(width='48%'))
    aim2 = widgets.BoundedIntText(description='Atom 2:', value=0,\
                                  min=0, layout=widgets.Layout(width='48%'))
    OK = widgets.Button(description='OK')
    geoi = widgets.IntText(value=0)
    
    output = widgets.VBox([title1, loc, files, resultshow])
    
    resultshow.on_click(intro_result)
    save1.on_click(pngsave)
    save2.on_click(csvsave)
    
    box_layout1 = widgets.Layout(display='flex', flew_flow='column',\
                                 overflow='scroll hidden', align_items='stretch',\
                                 border='solid', height='650px', width='26%')
    box_layout2 = widgets.Layout(display='flex', flew_flow='column',\
                                 overflow='scroll hidden', align_items='stretch',\
                                 border='solid', height='650px', width='37%')
    judgeoutput.layout = widgets.Layout(display='flex', flew_flow='column',\
                                        overflow='scroll hidden', align_items='stretch',\
                                        height='325px', width='100%')
    
    changecomboout = widgets.interactive_output(changecombo, {'loc': loc})
    filejudgeout = widgets.interactive_output(filejudge, {'files': files,\
                                                          'loc': loc})
    geoout = widgets.interactive_output(geoshow, {'aim1': aim1,\
                                                  'aim2': aim2, 'geoi': geoi, 'files': files})
    
    output = widgets.VBox([title1, loc, files, resultshow])
    output2.layout = box_layout2
    output3.layout = box_layout2
    frame1 = widgets.VBox([output, judgeoutput])
    frame1.layout=box_layout1
    
    resultshow.on_click(intro_result)
    save1.on_click(pngsave)
    save2.on_click(csvsave)
    
    display(widgets.HBox([frame1, output2, output3]))


import time as t
from .XyzToInp import XyzToInp
from .orca import orca

class multiorca:
    """
    The method to calculate several .inp files in one document consecutively by ORCA.
    multiorca([file1, file2, file3, ...], loc)
    file: Your .xyz file name.
    loc: File Location. The default is your current location.
    You can get the further information by .run.
    """
    def __init__(self, files=[], loc='./'):
        self.loc=loc
        self.files=files
        
    def run(self, orcaloc='./', method='B3LYP', basis_set='def2-SVP', opt=False,\
            freq=False, scan=False, external_force=False, ts=False,\
            aim=[0, 0], stretch=-1, scanstep=10, strain=-1, maxiter=-1,\
            maxcore=-1, corenum=1, electron=0, state=1):
        """
    The method to calculate several .inp files by ORCA and save file at the same location with .inp files.
    run(self, orcaloc='./', method='B3LYP', basis_set='def2-SVP', opt=False, freq=False, scan=False, external_force=False, ts=False, aim=[0, 0], stretch=-1, scanstep=10, strain=-1, maxiter=-1, maxcore=-1, corenum=1, electron=0, state=1)
    orcaloc: Your location of ORCA.
             If you have set ORCA from https://www.orcasoftware.de/tutorials_orca/first_steps/trouble_install.html#path-variable, 
             you can skip this key word.
    method: Your semiempirical/ab initio/DFT calculation methods. The default is B3LYP.
    basis_set: Your basis sets. The default is def2-SVP.
    opt: To show if you want to optimise your molecule. The default is False.
    freq: To show if you want to calculate the frequency of intermolecule system and thermodynamic data. The default is False.
    scan: To show if you want to scan the system. You must use key word 'aim', 'stretch' and 'scanstep' to express the site of external force, or it will cause 'ValueError'. 
        Herein, 'aim' is the site of external force, 'stretch' is the length you need to stretch (>0) or shrink(<0). (Unit: Å), and 'scanstep' is the number of your scan steps.
        The default is False.
    external_force: To show if you want to impose external force to the system. You must use key word 'aim' to express the site of external force AND key word 'strain' to express the force energy (Unit: nN), or it will cause 'ValueError'. The default is False.
    ts: To show if you want to calculate the transition state of the system. The default is False.
    maxiter: The max iteration steps of the optimization. It must perform after key word 'optimise' is True, or it will cause 'ValueError'. The default is based on ORCA.
    maxcore: The max space you need to calculate. The default is based on ORCA.
    corenum: The core you need to calculate. The default is 1.
    TIPS: To use key word 'maxcore' and 'corenum', please make sure you have set ORCA from following two websites.
            https://www.orcasoftware.de/tutorials_orca/first_steps/trouble_install.html#path-variable
            https://www.orcasoftware.de/tutorials_orca/first_steps/parallel.html
    electron: The charge of your systems. It might be positive(>0), negative(<0) or neutral(=0). The default is neutral.
    state: The state of your systems (singlet:1, doublet:2, triplet:3 ...). The default is singlet.
    TIPS: 1. Don't put the .inp to your ORCA installment file, it may disturb your work. 
    
          2. If you don't input key word 'orcaloc' while you don't set ORCA, you will only get an empty .out file in the document.
             There will be a warning: sh: orca: 'command not found'.
          3. If you have set key word 'maxcore' and 'corenum', you must input 'orcaloc'.
             See in https://www.orcasoftware.de/tutorials_orca/first_steps/parallel.html
          4. You can see the error of ORCA directly from the output.
    Example:
        Input:
            from MCPoly.orcaset import multiorca
            files=multiorca(['Atom1', 'Atom2'])
            orcaloc='./MCPoly/orca/'
            files.run(orcaloc=orcaloc, method='B3LYP', basis_set='def2-TZVP', opt=True, freq=True,\
                        maxcore=4096, corenum=8)
        Output:
            1. Atom1.xyz [Tue Apr 25 15:26:50 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 15:47:35 2023]
            
            2. Atom2.xyz [Tue Apr 25 15:47:36 2023]
            <<COMMAND LINES>>
            Program Completed.[Tue Apr 25 16:21:55 2023]
        """
        i = 1
        for file in self.files:
            print('{0}. '.format(i)+file+'.xyz '+'[{0}]'.format(t.ctime(t.time())))
            XyzToInp(file, fileloc=self.loc, saveloc=self.loc, method=method,\
                     basis_set=basis_set, opt=opt, freq=freq, scan=scan,\
                     external_force=external_force, ts=ts, aim=aim, stretch=stretch,\
                     scanstep=scanstep, strain=strain, maxiter=maxiter, maxcore=maxcore,\
                     corenum=corenum, electron=electron, state=state)
            orca(file, orcaloc, self.loc, self.loc)
            i = i + 1


import os
import re
import time as t

def orca(file, orcaloc='./', fileloc='./', saveloc='./'):
    """
    The method to calculate .inp file by ORCA.
    orca(file, orcaloc='./', fileloc='./', saveloc='./')
    file: Your .inp file name.
    orcaloc: Your location of ORCA.
             If you have set ORCA from https://www.orcasoftware.de/tutorials_orca/first_steps/trouble_install.html#path-variable, 
             you can skip this key word.
    fileloc: Your location of .inp file. The default is your current location.
    saveloc: Your location of .inp file. The default is your current location.
    Example:
        Input:
            from MCPoly.orcaset import XYZtoINP
            from MCPoly.orcaset import orca
            
            XYZtoINP('Atom2', method='B3LYP', basis_set='def2-TZVP', opt=True, freq=True, \
                        maxcore=4096, corenum=8)
            orca('Atom2', orcaloc='./MCPoly/orca/')
        Output:
            <<COMMAND LINES>>
            Program Completed.[Mon Apr 17 14:25:12 2023] -> See the current location to check files.
            
    TIPS: 1. Don't put the .inp to your ORCA installment file,  it may disturb your work. 
          2. If you don't input key word 'orcaloc' while you don't set ORCA,  you will only get an empty .out file in the document.
             There will be a warning: sh: orca: 'command not found'.
          3. If you have set key word 'maxcore' and 'corenum',  you must input 'orcaloc'.
             See in https://www.orcasoftware.de/tutorials_orca/first_steps/parallel.html
          4. You can see the error of ORCA directly from the output.
    """

    if orcaloc == './':
        os.system('orca {0}{1}.inp > {2}{1}.out'.format(fileloc, file, saveloc))
    else:
        path = os.getcwd() + '/'
        os.chdir(orcaloc)
        print(os.getcwd())
        if fileloc == './':
            fileloc = path
        if saveloc == './':
            saveloc = path
        os.system('./orca {0}{1}.inp > {2}{1}.out'.format(fileloc, file, saveloc))
        print('Program Completed.[{0}]\n'.format(t.ctime(t.time())))
        os.chdir(path[:-1])
    
    path = os.getcwd() + '/'
    os.chdir(fileloc)
    f = open(file+'.inp', 'r')
    for line in f:
        md = re.search('%md', line)
        if md:
            os.system('cp {0}{1}_dump.xyz {1}_dump.xyz'.format(orcaloc, file))
            os.system('rm {0}{1}_dump.xyz'.format(orcaloc, file))
    os.chdir(path)


import os
import re
from ase import Atoms
from ase.io import write

def DataToMolTxt(file, types = {0:0}, loc = './', savename = ''):
    """
    The method to change LAMMPS Data File into LAMMPS Molecule Text File.
    DataToMolTxt(file, types = {0:0, ...}, loc = './', savename = '')
    file: Your molecule system name on your .data file.
    types: Change the Atom Type number to suit the main LAMMPS input files.
    loc: File Location. The default is your current location.
    savename: Name of the saved XYZ File. The default is name of origin LAMMPS Data File.
    
    Example 1:
        Input:
            from MCPoly.lmpset import DATAtoXYZ
            DATAtomolTXT('Poly1')
            
        Output in Poly1.txt:
            #Polymer: BioPolymer1

            84 atoms
            85 bonds
            158 angles
            217 dihedrals
            48 impropers
            
            Coords
            
            1 1.3822 0.027 0.8364
                
                ...
       
            Types
    
            1 1
            2 2
            3 1
            4 2
            5 2
            6 3
            7 2
            8 4
            9 5
            
                ...
    
    Example 2:
        Input:
            from MCPoly.lmpset import DATAtomolTXT
            DataToMolTxt('Poly1', types = {1:11}, savename = 'Poly1_rami')
            
        Output in Poly1_rami:
            #Polymer: BioPolymer1

            84 atoms
            85 bonds
            158 angles
            217 dihedrals
            48 impropers
            
            Coords
            
            1 1.3822 0.027 0.8364
                
                ...
       
            Types
    
            1 11
            2 2
            3 11
            4 2
            5 2
            6 3
            7 2
            8 4
            9 5
            
                ...
    """
    opath = os.getcwd()
    os.chdir(loc)
    k = 0
    w1 = 0
    n1 = 0
    l = []
    pastline = ''
    real = [0, 0, 0]
    f = open(file+'.data', 'r')
    try:
        if savename == '':
            txt = open(file+'.txt', 'x')
        else:
            txt = open(savename+'.txt', 'x')
    except:
        if savename == '':
            txt = open(file+'.txt', 'w')
        else:
            txt = open(savename+'.txt', 'w')
    for line in f:
        del1 = re.search(r' types', line)
        del2 = re.search(r' [x-z]lo [x-z]hi', line)
        a1 = re.search(r' atoms', line)
        m1 = re.search(r'Masses', line)
        c1 = re.search(r'Atoms', line)
        #c2 = re.search(r'Bonds', line)
        #c3 = re.search(r'Angles', line)
        #c4 = re.search(r'Dihedrals', line)
        #c5 = re.search(r'Impropers', line)
        if del1:
            continue
        elif del2:
            continue
        elif pastline == line and line == '\n':
            continue
        elif a1:
            b1 = re.search(r'[0-9]+', line)
            num = b1.group(0)
            txt.write(line)
        elif c1:
            w1 = 1
        elif w1 == 1:
            l1 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if len(l1) == 7:
                l.append(l1)
                if l1[0] == num:
                    w1 = 2
        elif w1 == 2:
            txt.write('Coords\n\n')
            for i4 in range(len(l)):
                txt.write('{0} {1} {2} {3}\n'.format(i4+1, eval(l[i4][-3]),\
                                                     eval(l[i4][-2]), eval(l[i4][-1])))
            txt.write('\n')
            txt.write('Types\n\n')
            for i4 in range(len(l)):
                try:
                    ju = types[eval(l[i4][2])]
                    txt.write('{0} {1}\n'.format(i4+1, ju))
                except:
                    txt.write('{0} {1}\n'.format(i4+1, eval(l[i4][2])))
            txt.write('\n')
            txt.write('Charges\n\n')
            for i4 in range(len(l)):
                txt.write('{0} {1}\n'.format(i4+1, eval(l[i4][3])))
            txt.write('\n')
            w1 = 0
        elif n1 == 1:
            l0 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l0 == []:
                continue
            else:
                n1 = 2
        elif n1 == 2:
            l0 = re.findall(r'\-?[0-9]+\.?[0-9]*', line)
            if l0 == []:
                n1 = 0
                continue
            else:
                continue
        elif m1:
            n1 = 1
        else:
            txt.write(line)
        pastline = line
    f.close()
    txt.close()


from MCPoly.sscurve import single
from MCPoly.sscurve import multiple
from MCPoly.sscurve import multiple2
from MCPoly.sscurve import YModulus
import os
import re
import pytest

opath = os.getcwd()
os.chdir('./MCPoly/tests/data_sscurve')
try:
    os.system('rm Polymer1.png')
except:
    pass
try:
    os.system('rm Polymer1_Result.txt')
except:
    pass
try:
    os.system('rm Polymer3.png')
except:
    pass
try:
    os.system('rm Polymer3_Result.txt')
except:
    pass
try:
    os.system('rm Result.csv')
except:
    pass
try:
    os.system('rm Result.png')
except:
    pass
os.chdir(opath)

@pytest.fixture
def single1():
    return single(polymer='Polymer1', loc='./data_sscurve/')

def test_curve1(single1):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    single1.curve(2, 13, savefig=False, savedata=False, saveloc='./data_sscurve/', xx=True)
    i = 1
    for path in os.listdir(single1.loc):
        if os.path.isfile(os.path.join(single1.loc, path)):
            b = re.search('Polymer1_Result.txt', path)
            if b:
                i = 0
            c = re.search('Polymer1.png', path)
            if c:
                i = 0
    os.chdir(opath)
    assert i == 1

def test_autocurve1(single1):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    single1.autocurve(savefig=False, savedata=True, saveloc='./data_sscurve/', xx=True)
    i = 0
    j = 1
    for path in os.listdir(single1.loc):
        if os.path.isfile(os.path.join(single1.loc, path)):
            b = re.search('Polymer1_Result.txt', path)
            if b:
                i = 1
            c = re.search('Polymer1.png',path)
            if c:
                j = 0
    os.chdir(opath)
    assert i * j == 1
    
@pytest.fixture
def single3():
    return single(polymer='Polymer3', loc='./data_sscurve/')

def test_autocurve2(single3):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    single3.autocurve(savefig=False, saveloc='./data_sscurve/', xx=True)
    i = 0
    j = 1
    for path in os.listdir(single3.loc):
        if os.path.isfile(os.path.join(single3.loc, path)):
            b = re.search('Polymer3_Result.txt', path)
            if b:
                i = 1
#            c = re.search('Polymer3.png',path)
#            if c:
#                j = 1
    os.chdir(opath)
    assert i * j == 1
    
def test_multiple1():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    multiple(polymers=['Polymer1', 'Polymer2', 'Polymer3'],\
             loc='./data_sscurve/', savefig=False, savedata=False, xx=True)
    i = 1
    for path in os.listdir('./data_sscurve/'):
        if os.path.isfile(os.path.join('./data_sscurve/', path)):
#            b = re.search('Result.png', path)
#            if b:
#                i = 0
            c = re.search('Result.csv', path)
            if c:
                i = 0
    os.chdir(opath)
    assert i == 1

def test1_multiple2():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    multiple(loc='./data_sscurve/', savefig=False, needYM=False, xx=True)
    i = 1
    j = 0
    k = 1
    for path in os.listdir('./data_sscurve/'):
        if os.path.isfile(os.path.join('./data_sscurve/', path)):
            b = re.search('Result.png', path)
            if b:
                i = 0
            c = re.search('Result.csv', path)
            if c:
                j = 1
            d = re.search('ot.txt', path)
            if d:
                k = 0
    os.chdir(opath)
    assert i * j * k == 1

def test2_multiple2():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/data_sscurve/')
    f = open('Result.csv','r')
    i = 0
    for line in f:
        i = i + 1
    f.close()
    os.chdir(opath)
    assert i == 8

@pytest.fixture
def test_YModulus():
    os.system('rm ./MCPoly/tests/data_sscurve/Polymer1_Result.txt')
    return YModulus('./MCPoly/tests/data_sscurve/Polymer3_Result.txt') + 1

@pytest.fixture
def single1():
    return single(polymer='Polymer1', loc='./data_sscurve/')

def test_curve2(single1):
    os.system('rm ./MCPoly/tests/data_sscurve/Polymer1_Result.txt')
    os.system('rm ./MCPoly/tests/data_sscurve/Polymer1_Result.png')
    os.system('rm ./MCPoly/tests/data_sscurve/Polymer3_Result.txt')
    os.system('rm ./MCPoly/tests/data_sscurve/Result.csv')
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    single1.curve2(2, 13, savefig=False, savedata=False, saveloc='./data_sscurve/', xx=True)
    i = 1
    for path in os.listdir(single1.loc):
        if os.path.isfile(os.path.join(single1.loc, path)):
            b = re.search('Polymer1_Result.txt', path)
            if b:
                i = 0
            c = re.search('Polymer1.png', path)
            if c:
                i = 0
    os.chdir(opath)
    assert i == 1

def test_autocurve3(single1):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    single1.autocurve2(savefig=False, savedata=True, saveloc='./data_sscurve/', xx=True)
    i = 0
    j = 1
    for path in os.listdir(single1.loc):
        if os.path.isfile(os.path.join(single1.loc, path)):
            b = re.search('Polymer1_Result.txt', path)
            if b:
                i = 1
            c = re.search('Polymer1.png',path)
            if c:
                j = 0
    os.chdir(opath)
    assert i * j == 1

@pytest.fixture
def single3():
    return single(polymer='Polymer3', loc='./data_sscurve/')

def test_autocurve4(single3):
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    single3.autocurve2(savefig=False, saveloc='./data_sscurve/', xx=True)
    i = 0
    j = 1
    for path in os.listdir(single3.loc):
        if os.path.isfile(os.path.join(single3.loc, path)):
            b = re.search('Polymer3_Result.txt', path)
            if b:
                i = 1
#            c = re.search('Polymer3.png',path)
#            if c:
#                j = 1
    os.chdir(opath)
    assert i * j == 1
    
def test_multiple5():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    multiple2(polymers=['Polymer1', 'Polymer2', 'Polymer3'],\
              loc='./data_sscurve/', savefig=False, savedata=False, xx=True)
    i = 1
    for path in os.listdir('./data_sscurve/'):
        if os.path.isfile(os.path.join('./data_sscurve/', path)):
#            b = re.search('Result.png', path)
#            if b:
#                i = 0
            c = re.search('Result.csv', path)
            if c:
                i = 0
    os.chdir(opath)
    assert i == 1

def test1_multiple6():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/')
    multiple2(loc='./data_sscurve/', savefig=False, xx=True)
    i = 1
    j = 0
    k = 1
    for path in os.listdir('./data_sscurve/'):
        if os.path.isfile(os.path.join('./data_sscurve/', path)):
            b = re.search('Result.png', path)
            if b:
                i = 0
            c = re.search('Result.csv', path)
            if c:
                j = 1
            d = re.search('ot.txt', path)
            if d:
                k = 0
    os.chdir(opath)
    assert i * j * k == 1

def test2_multiple6():
    opath = os.getcwd()
    os.chdir('./MCPoly/tests/data_sscurve/')
    f = open('Result.csv','r')
    i = 0
    for line in f:
        i = i + 1
    f.close()
    os.chdir(opath)
    assert i == 8


from setuptools import setup

with open('README.md') as readme_file:
    README = readme_file.read()

setup_args = dict(
    name='MCPoly',
    version='0.6b3',
    description='Useful tools for Computational Chemistry for polymers',
    long_description_content_type="text/markdown",
    long_description=README,
    license="MIT",
    packages=['MCPoly','MCPoly.lmpset','MCPoly.moldraw','MCPoly.orcaset','MCPoly.sscurve','MCPoly.status','MCPoly.view3d','MCPoly.version','MCPoly.vis'],
    author='Omicron Fluor',
    author_email='cxs454@student.bham.ac.uk',
    keywords=['ORCA', 'Mechanical Property', 'Computational Chemistry'],
    url='https://github.com/Omicron-Fluor/MCPoly/',
)

install_requires = [
    'ase>=3.20.0',
    'ipywidgets>=8.0.0',
    'numpy>=1.22.0',
    'matplotlib>=3.6.0',
    'pandas>=1.4.0',
    'seaborn>=0.11.0',
    'py3dmol>=2.0.0',
    'rdkit>=2022.9.3',
    'scikit-learn>=1.1.2'
]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires,include_package_data=True)

